// $Id: CCompilador.cpp,v 1.36 2001/09/25 20:39:14 jpl Exp $

#include "CCompilador.hpp"

//
// CCOMPILADOR
//
CCompilador::CCompilador(const COpciones& op)
{
  opciones = new COpciones(op);

  
  NTemporal = 0;
  NEWhile = 0;

  entrada = new ifstream(op.Fichero_Compilar());
  analizador_lexico = new yyFlexLexer((istream*) entrada);
  tabla_simbolos = new CTablaSimbolos();

  Inicializar_NoTerminales(no_terminales);
  Inicializar_Terminales(terminales);
  Inicializar_SimbolosSemanticos(simbolos_semanticos);
  Inicializar_SimbolosGenCodigo(simbolos_gencodigo);
  Inicializar_TablaAS(tabla_as);
  Inicializar_Gramatica(gramatica);

  hay_main = false;
  hubo_error = false;

  pila_gramatical.push(NT_COMPILATION_UNIT);
}

//
// ~CCOMPILAR
//
CCompilador::~CCompilador(void)
{
  free(opciones);
  free(entrada);
  free(analizador_lexico);
  free(tabla_simbolos);
  
}

//
// COMPILAR
//
void
CCompilador::Compilar(void)
{
  token = analizador_lexico->yylex();
  if (!token)
    throw Lexico_Error_Ex("Ningn token encontrado.");

  if (opciones->Debug())
    cout<<"Token encontrado: "<<analizador_lexico->YYText()<<endl;

  // El algoritmo utilizado para el anlisis sintctico, es el 4.3 de
  // la pgina 192 del libro del dragn.
  Terminales::iterator itt;
  do {
    // Terminales.
    itt = find(terminales.begin(), terminales.end(), pila_gramatical.top());
    if (itt != terminales.end() ||  pila_gramatical.empty()) {
      if (pila_gramatical.top() == analizador_lexico->YYText()) {
	pila_gramatical.pop();

	if (opciones->Debug()) {
	  cout<<"Desapilado token: "<<analizador_lexico->YYText()<<endl;
	  Mostrar_Estado_Pila_Entrada();
	  Mostrar_Pila_Producciones();
	}

	token = analizador_lexico->yylex();

	if (!token)
	  if (pila_gramatical.top() == "lista_type_declaration")
	    Derivar (3);
	  else{
	    itt = find (no_terminales.begin(), no_terminales.end(), pila_gramatical.top());
	    if (itt != no_terminales.end())
	      throw Lexico_Error_Ex("Ningn token encontrado."); 
	  }

	if ( (token == IDENTIFICADOR) || (token == OCTAL) || (token == DECIMAL) ||
	     (token == HEXADECIMAL) || (token == CADENA_CARACTERES) )
	  terminales.insert(analizador_lexico->YYText());

	if (opciones->Debug() && token)
	  cout<<"Token encontrado: "<<analizador_lexico->YYText()<<endl;	
      }
      else	
	if (token) 
	  throw Sintactico_Error_Ex("Terminal no correcto.");
	else
	  throw Lexico_Error_Ex("Ningn token encontrado");
    }
    else {
      // Nbo Terminales.
      itt = find(no_terminales.begin(), no_terminales.end(), pila_gramatical.top());
      if (itt != no_terminales.end() || pila_gramatical.empty()) {
	int nprod = tabla_as[pila_gramatical.top()][token];
	if (opciones->Debug()) cout<<"Derivar por la produccion "<<nprod<<"."<<endl;
	if (nprod) { // Contiene producciones a analizar.
	  Derivar(nprod);
	  if (opciones->Debug()) { 
	    Mostrar_Estado_Pila_Entrada(); 
	    Mostrar_Pila_Producciones();
	  }
	}
	else 
	  if (pila_gramatical.top() == "lista_type_declaration")
	    //pila_gramatical.pop();
	    Derivar(3);
	  else {
	    pila_gramatical.pop();
	    hubo_error = true;
	    cerr<<opciones->Fichero_Compilar()<<":";
	    cerr<<analizador_lexico->lineno()<<":"<<"Error sintctico."<<endl;	    
	  }
      }
      else {
	// Smbolos semnticos.
	itt = find(simbolos_semanticos.begin(), simbolos_semanticos.end(),
		   pila_gramatical.top());
	if (itt != simbolos_semanticos.end() || pila_gramatical.empty()) {
	  if (opciones->Debug()) 
	    cout<<"Aplicar semntico: "<<pila_gramatical.top()<<endl;
	  try {
	    Analisis_Semantico(pila_gramatical.top());
	  }
	  catch ( Semantico_Error_Ex se ) {
	    hubo_error = true;
	    cerr<<opciones->Fichero_Compilar()<<":";
	    cerr<<analizador_lexico->lineno()<<":"<<se.what()<<endl;
	  }
	}
	else{ 
	  // Smbolos Generacin de Cdigo
	  itt = find(simbolos_gencodigo.begin(), simbolos_gencodigo.end(), 
		     pila_gramatical.top());
	  if (itt != simbolos_gencodigo.end() || pila_gramatical.empty()) {
	    if (opciones->Debug())
	      cout<<"Aplicar GenCodigo: "<<pila_gramatical.top()<<endl;
	    Generacion_Codigo(pila_gramatical.top());
	  } 
	  else {
	    hubo_error = true;
	    cerr<<opciones->Fichero_Compilar()<<":";
	    cerr<<analizador_lexico->lineno()<<":";
	    cerr<<"Smbolo "<<pila_gramatical.top()<<" no reconocido."<<endl;	    
	    pila_gramatical.pop();
	  }
	}
      }
    }
  } while ((token) || (!pila_gramatical.empty()));
  
  if (opciones->Debug()) { 
    Mostrar_Estado_Pila_Entrada(); 
    Mostrar_Pila_Producciones();
  }

  if (hubo_error)
    throw Hubo_Error_Ex();
}

//
// DERIVAR
//
void
CCompilador::Derivar(int nprod)
{
  // Se desapila el smbolo del que se deriva.
  pila_gramatical.pop();

  // Se apilan los smbolos de la produccin.
  switch(nprod) {
  case 11: // identifier
    pila_gramatical.push(analizador_lexico->YYText());
    pila_gramatical.push(G_11_0);
    pila_gramatical.push(S_11_0);
    break;
  case 94: // string
    pila_gramatical.push(analizador_lexico->YYText());
    pila_gramatical.push(G_94_0);
    pila_gramatical.push(S_94_0);
    break;
  case 91: // octal
    pila_gramatical.push(analizador_lexico->YYText());
    pila_gramatical.push(G_91_0);
    pila_gramatical.push(S_91_0);
    break;
  case 92: // decimal
    pila_gramatical.push(analizador_lexico->YYText());
    pila_gramatical.push(G_92_0);
    pila_gramatical.push(S_92_0);
    break;
  case 93: // hexadecimal
    pila_gramatical.push(analizador_lexico->YYText()); 
    pila_gramatical.push(G_93_0);
    pila_gramatical.push(S_93_0);
    break;
  default:
    for (unsigned int i = gramatica[nprod - 1].NSimbolos_Consecuente(); 
	 i > 0; i--)
      pila_gramatical.push(gramatica[nprod - 1][i - 1]);
  }
  
  // Se apila la produccin que genera la derivacin, para ms
  // adelante tratar sus atributos, slo si no se ha introducido ya.
  // Se puede haber introducido con anteridad, en el caso de los
  // atributos heredados, donde es necesario incluir la produccin,
  // esto es, tener su valor con anterioridad a la derivacin.
  if (pila_producciones.empty() || 
      pila_producciones.top().Antecedente() != gramatica[nprod - 1].Antecedente())
    pila_producciones.push(gramatica[nprod - 1]);
}

//
// MOSTRAR_ESTADO_PILA_ENTRADA
//
void
CCompilador::Mostrar_Estado_Pila_Entrada(void) const
{
  stack<string> p_aux = pila_gramatical;
  cout<<"[";
  while(!p_aux.empty()) {
    cout<<p_aux.top();
    p_aux.pop();
    if (!p_aux.empty()) cout<<", ";
  }
  cout<<"]"<<"["<<token<<" = "<<analizador_lexico->YYText()<<"]"<<endl;
}

//
// MOSTRAR_PILA_PRODUCCIONES
//
void
CCompilador::Mostrar_Pila_Producciones(void) const
{
  stack<CProduccion> pp = pila_producciones;
  
  cout<<"[";
  while(!pp.empty()) {
    cout<<pp.top().Antecedente();
    pp.pop();
    if (!pp.empty()) cout<<", ";
  }
  cout<<"]"<<endl;
}

//
// ANALISIS_SEMANTICO
//
void
CCompilador::Analisis_Semantico(const string& simbolo_semantico)
{
  pila_gramatical.pop();

  
  if (simbolo_semantico == S_1_0) {
    if (!hay_main)
      throw Semantico_Error_Ex(NO_HAY_MAIN);    
  }
  else if (simbolo_semantico == S_1_1) {

  }
  else if (simbolo_semantico == S_2_0) {
    
  }
  else if (simbolo_semantico == S_3_0) {
    
  }
  else if (simbolo_semantico == S_4_0) {
    
  }
  else if (simbolo_semantico == S_5_0) {
    tabla_simbolos->PopBloque();
  }
  else if (simbolo_semantico == S_5_1) {
    tabla_simbolos->PushBloque();
    CProduccion prod_aux_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_aux_lfd = gramatica[5];
    prod_aux_lfd.Set_Clase(prod_aux_id.Get_Valor());
    pila_producciones.push(prod_aux_lfd);
  }
  else if (simbolo_semantico == S_5_2) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);
    
    CProduccion prod_mod = Get_Primera_Produccion(NT_MODIFIER);
    CClaseDatos* cd = new CClaseDatos(prod_mod.Get_Acceso());
    
    CLexema lex(prod_id.Get_Valor(), L_CLASE, (void*)cd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }
  }
  else if (simbolo_semantico == S_6_0) {

  }
  else if (simbolo_semantico == S_6_1) {
    CProduccion prod_aux_lfd_a = Get_Primera_Produccion(NT_LISTA_FIELD_DECLARATION);
    CProduccion prod_aux_lfd_c = gramatica[5];
    prod_aux_lfd_c.Set_Clase(prod_aux_lfd_a.Get_Clase());
    pila_producciones.push(prod_aux_lfd_c);      
  }
  else if (simbolo_semantico == S_6_2) {
    CProduccion prod_aux_lfd = Get_Primera_Produccion(NT_LISTA_FIELD_DECLARATION);
    CProduccion prod_aux_fd = gramatica[11];
    prod_aux_fd.Set_Clase(prod_aux_lfd.Get_Clase());
    pila_producciones.push(prod_aux_fd);
  }
  else if (simbolo_semantico == S_7_0) {
    
  }
  else if (simbolo_semantico == S_8_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_MODIFIER);
    prod_aux.Set_Acceso(A_PUBLIC);
    Set_Primera_Produccion(prod_aux);
  }
  else if (simbolo_semantico == S_9_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_MODIFIER);
    prod_aux.Set_Acceso(A_PRIVATE);
    Set_Primera_Produccion(prod_aux);    
  }
  else if (simbolo_semantico == S_10_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_MODIFIER);
    prod_aux.Set_Acceso(A_PRIVATE);
    Set_Primera_Produccion(prod_aux);    
  }
  else if (simbolo_semantico == S_11_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_IDENTIFIER);
    prod_aux.Set_Valor(analizador_lexico->YYText());
    Set_Primera_Produccion(prod_aux);     
  }
  else if (simbolo_semantico == S_12_0) {
    
  }
  else if (simbolo_semantico == S_12_1) {
    CProduccion prod_aux_fd = Get_Primera_Produccion(NT_FIELD_DECLARATION);
    CProduccion prod_mod = Get_Primera_Produccion(NT_MODIFIER);
    CProduccion prod_type = Get_Primera_Produccion(NT_TYPE);
    CProduccion prod_aux_fdp = gramatica[13];

    prod_aux_fdp.Set_Acceso(prod_mod.Get_Acceso());
    prod_aux_fdp.Set_Tipo(prod_type.Get_Tipo());
    prod_aux_fdp.Set_Clase(prod_aux_fd.Get_Clase());
    pila_producciones.push(prod_aux_fdp);
  }
  else if (simbolo_semantico == S_13_0) {
    
  }
  else if (simbolo_semantico == S_14_0) {

  }
  else if (simbolo_semantico == S_14_1) {
    CProduccion prod_fdp = Get_Primera_Produccion(NT_FIELD_DECLARATION_PRIMA);
    CProduccion prod_cd = gramatica[98];

    prod_cd.Set_Acceso(prod_fdp.Get_Acceso());
    prod_cd.Set_Tipo(prod_fdp.Get_Tipo());
    prod_cd.Set_Clase(prod_fdp.Get_Clase());
    pila_producciones.push(prod_cd);    
  }
  else if (simbolo_semantico == S_15_0) {
    
  }
  else if (simbolo_semantico == S_15_1) {
    CProduccion prod_mod = Get_Primera_Produccion(NT_MODIFIER);
    CProduccion prod_type = Get_Primera_Produccion(NT_TYPE);
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_fdp = Get_Primera_Produccion(NT_FIELD_DECLARATION_PRIMA);
    
    CProduccion prod_fdpp = gramatica[15];
    prod_fdpp.Set_Acceso(prod_mod.Get_Acceso());
    prod_fdpp.Set_Tipo(prod_type.Get_Tipo());
    prod_fdpp.Set_Clase(prod_fdp.Get_Clase());
    prod_fdpp.Set_Valor(prod_id.Get_Valor());
    pila_producciones.push(prod_fdpp);
  }
  else if (simbolo_semantico == S_16_0) {
    
  }
  else if (simbolo_semantico == S_16_1) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_FIELD_DECLARATION_PRIMA_PRIMA);
    Tipo_Atributo atributo;

    atributo.acceso = prod_aux.Get_Acceso();
    atributo.tipo = prod_aux.Get_Tipo();
    atributo.id = prod_aux.Get_Valor();

    CLexema cl = tabla_simbolos->Recupera(prod_aux.Get_Clase());
    CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
    cld->Inserta_Atributo(atributo);
    cl.SetLexema(cl.GetID(), cl.GetTipo(), (void*)cld);
    tabla_simbolos->Modifica(cl);
  }
  else if (simbolo_semantico == S_17_0) {
    
  }
  else if (simbolo_semantico == S_17_1) {
    CProduccion prod_fdpp = Get_Primera_Produccion(NT_FIELD_DECLARATION_PRIMA_PRIMA);
    Tipo_Metodo metodo;

    metodo.acceso = prod_fdpp.Get_Acceso();
    metodo.tipo = prod_fdpp.Get_Tipo();
    metodo.id = prod_fdpp.Get_Valor();

    if (!hay_main)
      hay_main = metodo.id == "main";
    else if (hay_main && (metodo.id == "main"))
      throw Semantico_Error_Ex(SOLO_UN_MAIN);

    CLexema cl = tabla_simbolos->Recupera(prod_fdpp.Get_Clase());
    CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();

    if (cld->Es_Metodo(metodo.id))
      throw Semantico_Error_Ex(NO_SOBRECARGA);
    else {
      cld->Inserta_Metodo(metodo);
      cl.SetLexema(cl.GetID(), cl.GetTipo(), (void*)cld);
      tabla_simbolos->Modifica(cl);
    }

    CProduccion prod_md = gramatica[17];
    prod_md.Set_Valor(metodo.id);
    prod_md.Set_Clase(prod_fdpp.Get_Clase());
    pila_producciones.push(prod_md);
  }
  else if (simbolo_semantico == S_18_0) {
    tabla_simbolos->PopBloque();
  }
  else if (simbolo_semantico == S_18_1) {
    CProduccion prod_md = Get_Primera_Produccion(NT_METHOD_DECLARATION);
    CProduccion prod_mdf = gramatica[22];

    prod_mdf.Set_Valor(prod_md.Get_Valor());
    prod_mdf.Set_Clase(prod_md.Get_Clase());
    pila_producciones.push(prod_mdf);
  }
  else if (simbolo_semantico == S_18_2) {
    CProduccion prod_md = Get_Primera_Produccion(NT_METHOD_DECLARATION);
    CProduccion prod_opl = gramatica[18];

    prod_opl.Set_Valor(prod_md.Get_Valor());
    prod_opl.Set_Clase(prod_md.Get_Clase());
    pila_producciones.push(prod_opl);   

    tabla_simbolos->PushBloque();
  }  
  else if (simbolo_semantico == S_19_0) {
    
  }
  else if (simbolo_semantico == S_19_1) {
    CProduccion prod_opl = Get_Primera_Produccion(NT_OPCIONAL_PARAMETER_LIST);
    CProduccion prod_pl = gramatica[30];

    prod_pl.Set_Valor(prod_opl.Get_Valor());
    prod_pl.Set_Clase(prod_opl.Get_Clase());
    pila_producciones.push(prod_pl);
  }
  else if (simbolo_semantico == S_20_0) {
    
  }
  else if (simbolo_semantico == S_21_0) {
    
  }
  else if (simbolo_semantico == S_22_0) {
    
  }
  else if (simbolo_semantico == S_23_0) {
    
  }
  else if (simbolo_semantico == S_23_1) {
    CProduccion prod_mdf = Get_Primera_Produccion(NT_METHOD_DECLARATION_FINAL);
    CProduccion prod_sb = gramatica[34];

    prod_sb.Set_Valor(prod_mdf.Get_Valor());
    prod_sb.Set_Clase(prod_mdf.Get_Clase());
    pila_producciones.push(prod_sb);
  }
  else if (simbolo_semantico == S_24_0) {
    
  }
  else if (simbolo_semantico == S_25_0) {
    CProduccion prod_ts = Get_Primera_Produccion(NT_TYPE_SPECIFIER);
    CProduccion prod_t = Get_Primera_Produccion(NT_TYPE);
    prod_t.Set_Tipo(prod_ts.Get_Tipo());
    Set_Primera_Produccion(prod_t);    
  }
  else if (simbolo_semantico == S_26_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_TYPE_SPECIFIER);
    prod_aux.Set_Tipo(T_TD_VOID);
    Set_Primera_Produccion(prod_aux);    
  }
  else if (simbolo_semantico == S_27_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_TYPE_SPECIFIER);
    prod_aux.Set_Tipo(T_TD_BOOLEAN);
    Set_Primera_Produccion(prod_aux);    
  }
  else if (simbolo_semantico == S_28_0) {
    CProduccion prod_aux = Get_Primera_Produccion(NT_TYPE_SPECIFIER);
    prod_aux.Set_Tipo(T_TD_INT);
    Set_Primera_Produccion(prod_aux);    
  }
  else if (simbolo_semantico == S_29_0) {
    CProduccion prod_aux_class = Get_Primera_Produccion(NT_CLASS_NAME);
    CProduccion prod_aux_type = Get_Primera_Produccion(NT_TYPE_SPECIFIER);
    prod_aux_type.Set_Tipo(prod_aux_class.Get_Tipo());
    Set_Primera_Produccion(prod_aux_type);        
  }
  else if (simbolo_semantico == S_30_0) {
    CProduccion prod_aux_class = Get_Primera_Produccion(NT_CLASS_NAME);
    CProduccion prod_aux_identifier = Get_Primera_Produccion(NT_IDENTIFIER);
    prod_aux_class.Set_Tipo(prod_aux_identifier.Get_Valor());
    Set_Primera_Produccion(prod_aux_class);
  }
  else if (simbolo_semantico == S_31_0) {
    
  }
  else if (simbolo_semantico == S_31_1) {
    CProduccion prod_pl = Get_Primera_Produccion(NT_PARAMETER_LIST);
    CProduccion prod_p = Get_Primera_Produccion(NT_PARAMETER);
    Tipo_Parametro parametro;

    parametro.tipo = prod_p.Get_Tipo();
    parametro.id = prod_p.Get_Valor();

    CLexema cl = tabla_simbolos->Recupera(prod_pl.Get_Clase());
    CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
    cld->Inserta_Parametro(prod_pl.Get_Valor(), parametro);
    cl.SetLexema(cl.GetID(), cl.GetTipo(), (void*)cld);
    tabla_simbolos->Modifica(cl);

    CVariableDatos* vd = new CVariableDatos(prod_p.Get_Tipo(), "");
    CLexema lex(prod_p.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }   

    CProduccion prod_lpl = gramatica[31];
    prod_lpl.Set_Valor(prod_pl.Get_Valor());
    prod_lpl.Set_Clase(prod_pl.Get_Clase());
    pila_producciones.push(prod_lpl);
  }
  else if (simbolo_semantico == S_32_0) {
    
  }
  else if (simbolo_semantico == S_32_1) {
    CProduccion prod_lpl = Get_Primera_Produccion(NT_LISTA_PARAMETER_LIST);
    CProduccion prod_p = Get_Primera_Produccion(NT_PARAMETER);
    Tipo_Parametro parametro;

    parametro.tipo = prod_p.Get_Tipo();
    parametro.id = prod_p.Get_Valor();

    CLexema cl = tabla_simbolos->Recupera(prod_lpl.Get_Clase());
    CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
    cld->Inserta_Parametro(prod_lpl.Get_Valor(), parametro);
    cl.SetLexema(cl.GetID(), cl.GetTipo(), (void*)cld);
    tabla_simbolos->Modifica(cl);    

    CVariableDatos* vd = new CVariableDatos(prod_p.Get_Tipo(), "");
    CLexema lex(prod_p.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }       

    CProduccion prod_lpl2 = gramatica[31];
    prod_lpl2.Set_Valor(prod_lpl.Get_Valor());
    prod_lpl2.Set_Clase(prod_lpl.Get_Clase());
    pila_producciones.push(prod_lpl2);    
  }
  else if (simbolo_semantico == S_33_0) {
    
  }
  else if (simbolo_semantico == S_34_0) {
    
  }
  else if (simbolo_semantico == S_34_1) {
    CProduccion prod_p = Get_Primera_Produccion(NT_PARAMETER);
    CProduccion prod_type = Get_Primera_Produccion(NT_TYPE);
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    
    prod_p.Set_Valor(prod_id.Get_Valor());
    prod_p.Set_Tipo(prod_type.Get_Tipo());
    Set_Primera_Produccion(prod_p);
  }
  else if (simbolo_semantico == S_35_0) {
    tabla_simbolos->PopBloque();
  }
  else if (simbolo_semantico == S_35_1) {
    tabla_simbolos->PushBloque();

    CProduccion prod_sb = Get_Primera_Produccion(NT_STATMENT_BLOCK);
    CProduccion prod_ls = gramatica[35];
    
    prod_ls.Set_Valor(prod_sb.Get_Valor());
    prod_ls.Set_Clase(prod_sb.Get_Clase());
    pila_producciones.push(prod_ls);
  }
  else if (simbolo_semantico == S_36_0) {
    
  }
  else if (simbolo_semantico == S_36_1) {
    CProduccion prod_ls_a = Get_Primera_Produccion(NT_LISTA_STATMENT);
    CProduccion prod_ls_c = gramatica[35];

    prod_ls_c.Set_Valor(prod_ls_a.Get_Valor());
    prod_ls_c.Set_Clase(prod_ls_a.Get_Clase());
    pila_producciones.push(prod_ls_c);
  }
  else if (simbolo_semantico == S_36_2) {
    CProduccion prod_ls = Get_Primera_Produccion(NT_LISTA_STATMENT);
    CProduccion prod_s = gramatica[37];

    prod_s.Set_Valor(prod_ls.Get_Valor());
    prod_s.Set_Clase(prod_ls.Get_Clase());
    pila_producciones.push(prod_s);
  }
  else if (simbolo_semantico == S_37_0) {
    
  }
  else if (simbolo_semantico == S_38_0) {

  }
  else if (simbolo_semantico == S_38_1) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_s = Get_Primera_Produccion(NT_STATMENT);
    CProduccion prod_ps = gramatica[104];

    prod_ps.Set_Valor(prod_id.Get_Valor());

    CLexema lex = tabla_simbolos->Recupera(prod_id.Get_Valor());

    if (lex.GetTipo() == L_VARIABLE || lex.GetTipo() == L_CONSTANTE) {
      CVariableDatos* vd = (CVariableDatos*)lex.GetDatos();
      prod_ps.Set_Tipo(vd->GetTipoDeDatos());
    } 
    else if (lex.GetTipo() == L_CLASE)
      prod_ps.Set_Tipo(prod_id.Get_Valor());

    prod_ps.Set_Clase(prod_s.Get_Clase());
    pila_producciones.push(prod_ps);
  }    
  else if (simbolo_semantico == S_39_0) {
    
  }
  else if (simbolo_semantico == S_39_1) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CVariableDatos* vd = new CVariableDatos(T_TD_VOID, "");
    CLexema lex(prod_id.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }

    CProduccion prod_vd = gramatica[47];
    prod_vd.Set_Tipo(T_TD_VOID);
    pila_producciones.push(prod_vd);
  }
  else if (simbolo_semantico == S_40_0) {
    
  }
  else if (simbolo_semantico == S_40_1) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CVariableDatos* vd = new CVariableDatos(T_TD_INT, "");
    CLexema lex(prod_id.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }     

    CProduccion prod_vd = gramatica[47];
    prod_vd.Set_Tipo(T_TD_INT);
    pila_producciones.push(prod_vd);   
  }
  else if (simbolo_semantico == S_41_0) {
    
  }
  else if (simbolo_semantico == S_41_1) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CVariableDatos* vd = new CVariableDatos(T_TD_BOOLEAN, "");
    CLexema lex(prod_id.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    } 

    CProduccion prod_vd = gramatica[47];
    prod_vd.Set_Tipo(T_TD_BOOLEAN);
    pila_producciones.push(prod_vd);       
  }
  else if (simbolo_semantico == S_42_0) {
    
  }
  else if (simbolo_semantico == S_42_1) {
    CProduccion prod_s = Get_Primera_Produccion(NT_STATMENT);
    CProduccion prod_sb = gramatica[34];

    prod_sb.Set_Valor(prod_s.Get_Valor());
    prod_sb.Set_Clase(prod_s.Get_Clase());
    pila_producciones.push(prod_sb);
  }
  else if (simbolo_semantico == S_43_0) {
    
  }
  else if (simbolo_semantico == S_43_1) {
    CProduccion prod_s = Get_Primera_Produccion(NT_STATMENT);
    CProduccion prod_ds = gramatica[97];

    prod_ds.Set_Valor(prod_s.Get_Valor());
    prod_ds.Set_Clase(prod_s.Get_Clase());
    pila_producciones.push(prod_ds);    
  }
  else if (simbolo_semantico == S_44_0) {
    CProduccion prod_s = Get_Primera_Produccion(NT_STATMENT);
    CProduccion prod_oe = Get_Primera_Produccion(NT_OPCIONAL_EXPRESION);

    CLexema lex = tabla_simbolos->Recupera(prod_s.Get_Clase());
    CClaseDatos* cd = (CClaseDatos*)lex.GetDatos();

    if (cd->Get_Tipo_Metodo(prod_s.Get_Valor()) != prod_oe.Get_Tipo())
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);
  }
  else if (simbolo_semantico == S_44_1) {
    CProduccion prod_s = Get_Primera_Produccion(NT_STATMENT);
    CProduccion prod_oe = gramatica[45];

    prod_oe.Set_Valor(prod_s.Get_Valor());
    prod_oe.Set_Clase(prod_s.Get_Clase());
    pila_producciones.push(prod_oe);      
  }
  else if (simbolo_semantico == S_45_0) {
    
  }
  else if (simbolo_semantico == S_46_0) {
    CProduccion prod_oe = Get_Primera_Produccion(NT_OPCIONAL_EXPRESION);
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);

    prod_oe.Set_Tipo(prod_e.Get_Tipo());
    Set_Primera_Produccion(prod_oe);
  }
  else if (simbolo_semantico == S_46_1) {
    CProduccion prod_oe = Get_Primera_Produccion(NT_OPCIONAL_EXPRESION);
    CProduccion prod_e = gramatica[54];

    prod_e.Set_Valor(prod_oe.Get_Valor());
    prod_e.Set_Clase(prod_oe.Get_Clase());
    pila_producciones.push(prod_e);    
  }
  else if (simbolo_semantico == S_47_0) {
    
  }
  else if (simbolo_semantico == S_48_0) {
    
  }
  else if (simbolo_semantico == S_48_1) {
    CProduccion prod_vdon = Get_Primera_Produccion(NT_VARIABLE_DECLARATION);
    CProduccion prod_lvd = gramatica[48];

    prod_lvd.Set_Tipo(prod_vdon.Get_Tipo());
    pila_producciones.push(prod_lvd);
  }
  else if (simbolo_semantico == S_48_2) {
    CProduccion prod_vdon = Get_Primera_Produccion(NT_VARIABLE_DECLARATION);
    CProduccion prod_vdor = gramatica[50];

    prod_vdor.Set_Tipo(prod_vdon.Get_Tipo());
    pila_producciones.push(prod_vdor);    
  }
  else if (simbolo_semantico == S_49_0) {
    
  }
  else if (simbolo_semantico == S_49_1) {
    CProduccion prod_lvd_a = Get_Primera_Produccion(NT_LISTA_VARIABLE_DECLARATOR);
    CProduccion prod_lvd_c = gramatica[48];

    prod_lvd_c.Set_Tipo(prod_lvd_a.Get_Tipo());
    pila_producciones.push(prod_lvd_c);    
  }
  else if (simbolo_semantico == S_49_2) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CProduccion prod_lvd = Get_Primera_Produccion(NT_LISTA_VARIABLE_DECLARATOR);
    CVariableDatos* vd = new CVariableDatos(prod_lvd.Get_Tipo(), "");
    CLexema lex(prod_id.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }

    CProduccion prod_vd = gramatica[51];
    prod_vd.Set_Tipo(prod_lvd.Get_Tipo());
    pila_producciones.push(prod_vd);
  }
  else if (simbolo_semantico == S_50_0) {
    
  }
  else if (simbolo_semantico == S_51_0) {
    
  }
  else if (simbolo_semantico == S_51_1) {
    CProduccion prod_vd = Get_Primera_Produccion(NT_VARIABLE_DECLARATOR);
    CProduccion prod_ovi = gramatica[51];

    prod_ovi.Set_Tipo(prod_vd.Get_Tipo());
    pila_producciones.push(prod_ovi);
  }
  else if (simbolo_semantico == S_52_0) {
    CProduccion ovi = Get_Primera_Produccion(NT_OPCIONAL_VARIABLE_INITIALIZER);
    CProduccion vi = Get_Primera_Produccion(NT_VARIABLE_INITIALIZER);

    if (ovi.Get_Tipo() != vi.Get_Tipo())
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);
  }
  else if (simbolo_semantico == S_53_0) {
    
  }
  else if (simbolo_semantico == S_54_0) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_vi = Get_Primera_Produccion(NT_VARIABLE_INITIALIZER);

    prod_vi.Set_Tipo(prod_e.Get_Tipo());
    Set_Primera_Produccion(prod_vi);
  }
  else if (simbolo_semantico == S_55_0) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    // DEBUG --------------
//    cout<<"**** "<<prod_e1.Get_Valor()<<":"<<prod_e1.Get_Tipo()<<" ****"<<endl;
    // DEBUG --------------
    prod_e.Set_Tipo(prod_ep.Get_Tipo());

    Set_Primera_Produccion(prod_e);
  }
  else if (simbolo_semantico == S_55_1) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    CProduccion prod_ep = gramatica[55];
    // DEBUG --------------
//    cout<<"**** "<<prod_e1.Get_Valor()<<":"<<prod_e1.Get_Tipo()<<" ****"<<endl;
    // DEBUG --------------
    prod_ep.Set_Valor(prod_e1.Get_Valor());
    prod_ep.Set_Clase(prod_e.Get_Clase());
    prod_ep.Set_Tipo(prod_e1.Get_Tipo());
    pila_producciones.push(prod_ep);
  }
  else if (simbolo_semantico == S_55_2) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_e1 = gramatica[61];

    prod_e1.Set_Valor(prod_e.Get_Valor());
    prod_e1.Set_TProd(prod_e.Get_Clase());
    prod_e1.Set_Clase(prod_e.Get_Clase());
    pila_producciones.push(prod_e1);    
  }
  else if (simbolo_semantico == S_56_0) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    prod_e.Set_Tipo(T_TD_INT);
    Set_Primera_Produccion(prod_e);
  }
  else if (simbolo_semantico == S_56_1) {
    CProduccion prod_ep_a = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_ep_c = gramatica[55];

    prod_ep_c.Set_Valor(prod_ep_a.Get_Valor());
    prod_ep_c.Set_Tipo(prod_ep_a.Get_Tipo());
    prod_ep_c.Set_Clase(prod_ep_a.Get_Clase());
    pila_producciones.push(prod_ep_c);    
  }
  else if (simbolo_semantico == S_57_0) {
    
  }
  else if (simbolo_semantico == S_57_1) {
    CProduccion prod_ep_a = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_ep_c = gramatica[55];

    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    prod_ep_c.Set_Valor(prod_ep_a.Get_Valor());
    prod_ep_c.Set_Valor(prod_e1.Get_Tipo());
    prod_ep_c.Set_Clase(prod_ep_a.Get_Clase());
    pila_producciones.push(prod_ep_c);

    if ((T_TD_INT != prod_e1.Get_Tipo()) ||
	(prod_e1.Get_Tipo() != prod_ep_a.Get_Tipo()))
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);

    prod_ep_a.Set_Tipo(T_TD_INT);
    Set_Primera_Produccion(prod_ep_a);
  }
  else if (simbolo_semantico == S_57_2) {
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_e1 = gramatica[61];

    prod_e1.Set_Valor(prod_ep.Get_Valor());
    prod_e1.Set_Clase(prod_ep.Get_Clase());
    pila_producciones.push(prod_e1);        
  }
  else if (simbolo_semantico == S_58_0) {
    
  }
  else if (simbolo_semantico == S_58_1) {    
    CProduccion prod_ep_a = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_ep_c = gramatica[55];

    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    prod_ep_c.Set_Valor(prod_ep_a.Get_Valor());
    prod_ep_c.Set_Tipo(prod_e1.Get_Tipo());
    prod_ep_c.Set_Clase(prod_ep_a.Get_Clase());
    pila_producciones.push(prod_ep_c);

    if (prod_e1.Get_Tipo() != prod_ep_a.Get_Tipo())
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);

    prod_ep_a.Set_Tipo(T_TD_BOOLEAN);
    Set_Primera_Produccion(prod_ep_a);
  }
  else if (simbolo_semantico == S_58_2) {
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_e1 = gramatica[61];

    prod_e1.Set_Valor(prod_ep.Get_Valor());
    prod_e1.Set_Clase(prod_ep.Get_Clase());
    pila_producciones.push(prod_e1);      
  }
  else if (simbolo_semantico == S_59_0) {
    
  }
  else if (simbolo_semantico == S_59_1) {
    CProduccion prod_ep_a = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_ep_c = gramatica[55];

    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    prod_ep_c.Set_Valor(prod_ep_a.Get_Valor());
    prod_ep_c.Set_Tipo(prod_e1.Get_Tipo());
    prod_ep_c.Set_Clase(prod_ep_a.Get_Clase());
    pila_producciones.push(prod_ep_c);

    if ((T_TD_BOOLEAN != prod_e1.Get_Tipo()) ||
	(prod_e1.Get_Tipo() != prod_ep_a.Get_Tipo()))
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);

    prod_ep_a.Set_Tipo(T_TD_BOOLEAN);
    Set_Primera_Produccion(prod_ep_a);
  }
  else if (simbolo_semantico == S_59_2) {
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CProduccion prod_e1 = gramatica[61];

    prod_e1.Set_Valor(prod_ep.Get_Valor());
    prod_e1.Set_Clase(prod_ep.Get_Clase());
    pila_producciones.push(prod_e1);      
  }
  else if (simbolo_semantico == S_60_0) {

  }
  else if (simbolo_semantico == S_60_1) {
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CLexema lex_var = tabla_simbolos->Recupera(prod_ep.Get_Valor());
    if (lex_var.GetTipo() != L_VARIABLE)
      throw Semantico_Error_Ex(NO_ES_UNA_VARIABLE);
    
    CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
    CLexema lex_cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());

    // DEBUG --------------
    //cout<<"**** "<<vd->GetTipoDeDatos()<<" ****"<<endl;
    // DEBUG --------------

    if (lex_cl.GetTipo() != L_CLASE)
      throw Semantico_Error_Ex(NO_ES_UNA_CLASE);

    CClaseDatos* cld = (CClaseDatos*)lex_cl.GetDatos();
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);    

    prod_ep.Set_Tipo(prod_e1.Get_Tipo());
    Set_Primera_Produccion(prod_ep);

    if (!cld->Es_Metodo(prod_e1.Get_Valor()) &&
	!cld->Es_Atributo(prod_e1.Get_Valor()))
      throw Semantico_Error_Ex(MIEMBRO_NO_DECLARADO);
    
    if (cld->Es_Metodo(prod_e1.Get_Valor()) &&
	cld->Es_Metodo_Privado(prod_e1.Get_Valor()))
      throw Semantico_Error_Ex(METODO_PRIVADO);
    else if (cld->Es_Atributo(prod_e1.Get_Valor()) &&
	     cld->Es_Atributo_Privado(prod_e1.Get_Valor()))
      throw Semantico_Error_Ex(ATRIBUTO_PRIVADO);    

    CProduccion prod_ep_c = gramatica[55];
    prod_ep_c.Set_Valor(prod_ep.Get_Valor());
    prod_ep_c.Set_Clase(prod_ep.Get_Clase());
    pila_producciones.push(prod_ep_c);
  }
  else if (simbolo_semantico == S_60_2) {
    CProduccion prod_ep = Get_Primera_Produccion(NT_EXPRESION_PRIMA);
    CLexema lex_var = tabla_simbolos->Recupera(prod_ep.Get_Valor());
    if (lex_var.GetTipo() != L_VARIABLE)
      throw Semantico_Error_Ex(NO_ES_UNA_VARIABLE);
    
    CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
    CLexema lex_cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());

    if (lex_cl.GetTipo() != L_CLASE)
      throw Semantico_Error_Ex(NO_ES_UNA_CLASE);

    CProduccion prod_e1 = gramatica[61];
    prod_e1.Set_Valor(prod_ep.Get_Valor());
    prod_e1.Set_TProd(vd->GetTipoDeDatos());
    prod_e1.Set_Clase(vd->GetTipoDeDatos());
    pila_producciones.push(prod_e1);
  }
  else if (simbolo_semantico == S_61_0) {
    
  }
  else if (simbolo_semantico == S_62_0) {
    if (opciones->Debug()) {
      Mostrar_Estado_Pila_Entrada();
      Mostrar_Pila_Producciones();
    }
    
    CProduccion prod_e1p = Get_Primera_Produccion(NT_EXPRESION_1_PRIMA);
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);

    prod_e1.Set_Tipo(prod_e1p.Get_Tipo());
    Set_Primera_Produccion(prod_e1);
  }
  else if (simbolo_semantico == S_62_1) {
    CProduccion prod_e2 = Get_Primera_Produccion(NT_EXPRESION_2);
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);

    if (prod_e2.Get_Tipo() == T_TD_BOOLEAN ||
	prod_e2.Get_Tipo() == T_TD_INT ||
	prod_e2.Get_Tipo() == T_TD_CADENA)
      prod_e1.Set_Tipo(prod_e2.Get_Tipo());
    else {
      // DEBUG ----------------
//      cout<<"**** "<<prod_e1.Get_Valor()<<":"<<prod_e2.Get_Tipo()<<"**** "<<endl;
//       cout<<"**** "<<prod_e2.Get_Valor()<<"**** "<<endl;
      // DEBUG ----------------
      if (tabla_simbolos->Busca(prod_e2.Get_Valor())) {
	CLexema lex = tabla_simbolos->Recupera(prod_e2.Get_Valor());
	if (lex.GetTipo() == L_VARIABLE) {
	  CVariableDatos* vd = (CVariableDatos*)lex.GetDatos();
	  prod_e1.Set_Tipo(vd->GetTipoDeDatos());
	}
	else if (lex.GetTipo() == L_CONSTANTE) {
	  CConstanteDatos* cd = (CConstanteDatos*)lex.GetDatos();
	  prod_e1.Set_Tipo(cd->GetTipoDeDatos());
	}
      }
      else if (tabla_simbolos->Busca(prod_e1.Get_Clase())) {
	CLexema lex = tabla_simbolos->Recupera(prod_e1.Get_Clase());
	CClaseDatos* cd = (CClaseDatos*)lex.GetDatos();
	if (cd->Es_Metodo(prod_e2.Get_Valor())){
	  prod_e1.Set_Tipo(cd->Get_Tipo_Metodo(prod_e2.Get_Valor()));
	  // DEBUG ----------------
// 	  cout<<"**** "<<prod_e2.Get_Valor()<<":"<<prod_e1.Get_Tipo()<<" ****"<<endl;
 	  // DEBUG ----------------	  
	}
	else
	  throw Semantico_Error_Ex(VARIABLE_NO_DECLARADA);
      }
    }
        
    prod_e1.Set_Valor(prod_e2.Get_Valor());
    Set_Primera_Produccion(prod_e1);

    CProduccion prod_e1p = gramatica[62];
    prod_e1p.Set_Valor(prod_e2.Get_Valor());
    prod_e1p.Set_Tipo(prod_e1.Get_Tipo());
    prod_e1p.Set_Clase(prod_e2.Get_Clase());
    pila_producciones.push(prod_e1p);
  }
  else if (simbolo_semantico == S_62_2) {
    CProduccion prod_e1 = Get_Primera_Produccion(NT_EXPRESION_1);
    CProduccion prod_e2 = gramatica[64];

    prod_e2.Set_Valor(prod_e1.Get_Valor());
    prod_e2.Set_Clase(prod_e1.Get_Clase());
    pila_producciones.push(prod_e2);
  }
  else if (simbolo_semantico == S_63_0) {
    // CProduccion prod_e1p_a = Get_Segunda_Produccion(NT_EXPRESION_1_PRIMA);
//     CProduccion prod_e1p_c = Get_Primera_Produccion(NT_EXPRESION_1_PRIMA);
//     if (opciones->Debug()) {
// 	Mostrar_Estado_Pila_Entrada();
// 	Mostrar_Pila_Producciones();
//       }
//     prod_e1p_a.Set_Tipo(prod_e1p_c.Get_Tipo());
//     Set_Segunda_Produccion(prod_e1p_a);
//     if (opciones->Debug()) {
// 	Mostrar_Estado_Pila_Entrada();
// 	Mostrar_Pila_Producciones();
//       }
  }
  else if (simbolo_semantico == S_63_1) {
    CProduccion prod_e1p_a = Get_Primera_Produccion(NT_EXPRESION_1_PRIMA);
    CProduccion prod_e1p_c = gramatica[62];

    prod_e1p_c.Set_Valor(prod_e1p_a.Get_Valor());
    prod_e1p_c.Set_Clase(prod_e1p_a.Get_Clase());
    pila_producciones.push(prod_e1p_c);
  }
  else if (simbolo_semantico == S_63_2) {
    CProduccion prod_e1p = Get_Primera_Produccion(NT_EXPRESION_1_PRIMA);
    CProduccion prod_e3 = gramatica[68];

    prod_e3.Set_Valor(prod_e1p.Get_Valor());
    prod_e3.Set_Clase(prod_e1p.Get_Clase());
    pila_producciones.push(prod_e3);
  }
  else if (simbolo_semantico == S_64_0) {
    if (opciones->Debug()) {
      Mostrar_Estado_Pila_Entrada();
      Mostrar_Pila_Producciones();
    }
  }
  else if (simbolo_semantico == S_65_0) {
    CProduccion prod_e2 = Get_Primera_Produccion(NT_EXPRESION_2);
    CProduccion prod_cl = Get_Primera_Produccion(NT_CONSTANTE_LOGICA);

    prod_e2.Set_Tipo(prod_cl.Get_Tipo());
    Set_Primera_Produccion(prod_e2);
  }
  else if (simbolo_semantico == S_66_0) {
    CProduccion prod_e2 = Get_Primera_Produccion(NT_EXPRESION_2);
    CProduccion prod_le = Get_Primera_Produccion(NT_LITERAL_EXPRESION);

    prod_e2.Set_Tipo(prod_le.Get_Tipo());
    Set_Primera_Produccion(prod_e2);
  }
  else if (simbolo_semantico == S_67_0) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_e2 = Get_Primera_Produccion(NT_EXPRESION_2);
    
    prod_e2.Set_Valor(prod_id.Get_Valor());
//    prod_e2.Set_Tipo(prod_id.Get_Valor());
    Set_Primera_Produccion(prod_e2);
  }
  else if (simbolo_semantico == S_68_0) {
    CProduccion prod_e2 = Get_Primera_Produccion(NT_EXPRESION_2);
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);

    prod_e2.Set_Tipo(prod_e.Get_Tipo());
    Set_Primera_Produccion(prod_e2);
  }
  else if (simbolo_semantico == S_69_0) {
    
  }
  else if (simbolo_semantico == S_69_0) {

  }
  else if (simbolo_semantico == S_69_1) {
    CProduccion prod_e3 = Get_Primera_Produccion(NT_EXPRESION_3);
    CProduccion prod_ae = gramatica[84];

    prod_ae.Set_Valor(prod_e3.Get_Valor());
    prod_ae.Set_Clase(prod_e3.Get_Clase());
    pila_producciones.push(prod_ae);    
  }
  else if (simbolo_semantico == S_70_0) {
    
  }
  else if (simbolo_semantico == S_71_0) {
    
  }
  else if (simbolo_semantico == S_72_0) {
    
  }
  else if (simbolo_semantico == S_73_0) {
    
  }
  else if (simbolo_semantico == S_74_0) {
    
  }
  else if (simbolo_semantico == S_75_0) {
    
  }
  else if (simbolo_semantico == S_76_0) {
    
  }
  else if (simbolo_semantico == S_77_0) {
    
  }
  else if (simbolo_semantico == S_78_0) {
    
  }
  else if (simbolo_semantico == S_79_0) {
    
  }
  else if (simbolo_semantico == S_80_0) {
    
  }
  else if (simbolo_semantico == S_81_0) {
    
  }
  else if (simbolo_semantico == S_82_0) {
    
  }
  else if (simbolo_semantico == S_83_0) {
    
  }
  else if (simbolo_semantico == S_84_0) {
    
  }
  else if (simbolo_semantico == S_85_0) {
    
  }
  else if (simbolo_semantico == S_85_1) {
    CProduccion prod_ae = Get_Primera_Produccion(NT_AGRUPA_EXPRESION);
    CProduccion prod_oa = gramatica[86];

    prod_oa.Set_Valor(prod_ae.Get_Valor());
    prod_oa.Set_Clase(prod_ae.Get_Clase());
    pila_producciones.push(prod_oa);
  }
  else if (simbolo_semantico == S_86_0) {
    
  }
  else if (simbolo_semantico == S_87_0) {
    
  }
  else if (simbolo_semantico == S_87_1) {
    CProduccion prod_oa = Get_Primera_Produccion(NT_OPCIONAL_ARGLIST);
    CProduccion prod_a = gramatica[94];

    if (prod_oa.Get_Valor() == "read")
      throw Semantico_Error_Ex(PARAMETROS_INCORRECTOS);

    prod_a.Set_Valor(prod_oa.Get_Valor());
    prod_a.Set_Clase(prod_oa.Get_Clase());
    pila_producciones.push(prod_a);
  }
  else if (simbolo_semantico == S_88_0) {
    
  }
  else if (simbolo_semantico == S_89_0) {
    CProduccion prod_le = Get_Primera_Produccion(NT_LITERAL_EXPRESION);
    CProduccion prod_il = Get_Primera_Produccion(NT_INTEGER_LITERAL);

    prod_le.Set_Valor(prod_il.Get_Valor());
    prod_le.Set_Tipo(prod_il.Get_Tipo());
    Set_Primera_Produccion(prod_le);
  }
  else if (simbolo_semantico == S_90_0) {
    CProduccion prod_le = Get_Primera_Produccion(NT_LITERAL_EXPRESION);
    prod_le.Set_Tipo(T_TD_CADENA);
    prod_le.Set_Valor(analizador_lexico->YYText());
    Set_Primera_Produccion(prod_le);
  }
  else if (simbolo_semantico == S_91_0) {
    int octal;
    char cadena_entero[10];

    sscanf(analizador_lexico->YYText(), "%o", &octal);
    sprintf(cadena_entero, "%d", octal);

    CProduccion prod_il = Get_Primera_Produccion(NT_INTEGER_LITERAL);
    prod_il.Set_Tipo(T_TD_INT);
    prod_il.Set_Valor(cadena_entero);
    Set_Primera_Produccion(prod_il);   
  }
  else if (simbolo_semantico == S_92_0) {
    int decimal;
    char cadena_entero[10];

    sscanf(analizador_lexico->YYText(), "%d", &decimal);
    sprintf(cadena_entero, "%d", decimal);

    CProduccion prod_il = Get_Primera_Produccion(NT_INTEGER_LITERAL);
    prod_il.Set_Tipo(T_TD_INT);
    prod_il.Set_Valor(cadena_entero);
    Set_Primera_Produccion(prod_il);       
  }
  else if (simbolo_semantico == S_93_0) {
    int hex;
    char cadena_entero[10];

    sscanf(analizador_lexico->YYText(), "%x", &hex);
    sprintf(cadena_entero, "%d", hex);

    CProduccion prod_il = Get_Primera_Produccion(NT_INTEGER_LITERAL);
    prod_il.Set_Tipo(T_TD_INT);
    prod_il.Set_Valor(cadena_entero);
    Set_Primera_Produccion(prod_il);       
  }
  else if (simbolo_semantico == S_94_0) {
    
  }
  else if (simbolo_semantico == S_95_0) {
    
  }
  else if (simbolo_semantico == S_95_1) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_a = Get_Primera_Produccion(NT_ARGLIST);

    if (tabla_simbolos->Busca(prod_a.Get_Clase())) {
      CLexema lex = tabla_simbolos->Recupera(prod_a.Get_Clase());
      CClaseDatos* cd = (CClaseDatos*)lex.GetDatos();
      
      if (cd->Es_Metodo(prod_a.Get_Valor())) {
	Tipo_Parametro para = cd->Get_NParametro(prod_a.Get_Valor(), 1);
	if (para.tipo != prod_e.Get_Tipo())
	  throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);
      }
    }

    CProduccion prod_le = gramatica[95];
    prod_le.Set_Clase(prod_a.Get_Clase());
    prod_le.Set_Valor(prod_a.Get_Valor());
    prod_le.Set_NProduccion(1);
    pila_producciones.push(prod_le);
  }
  else if (simbolo_semantico == S_95_2) {
    CProduccion prod_a = Get_Primera_Produccion(NT_ARGLIST);
    CProduccion prod_e = gramatica[54];

    prod_e.Set_Valor(prod_a.Get_Valor());
    prod_e.Set_Clase(prod_a.Get_Clase());
    pila_producciones.push(prod_e);
  }
  else if (simbolo_semantico == S_96_0) {

  }
  else if (simbolo_semantico == S_96_1) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_le = Get_Primera_Produccion(NT_LISTA_EXPRESION);    

    if (tabla_simbolos->Busca(prod_le.Get_Clase())) {
      CLexema lex = tabla_simbolos->Recupera(prod_le.Get_Clase());
      CClaseDatos* cd = (CClaseDatos*)lex.GetDatos();
      
      if (cd->Es_Metodo(prod_le.Get_Valor())) {
	Tipo_Parametro para = cd->Get_NParametro(prod_le.Get_Valor(),
						 prod_le.Get_NProduccion());
	if (para.tipo != prod_e.Get_Tipo())
	  throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);

	CVariableDatos* vd = new CVariableDatos(para.tipo, "");
	CLexema lex(para.id, L_VARIABLE, (void*)vd);
	tabla_simbolos->Inserta(lex);
      }
    }

    CProduccion prod_le_c = gramatica[95];
    prod_le_c.Set_Clase(prod_le.Get_Clase());
    prod_le_c.Set_Valor(prod_le.Get_Valor());
    prod_le_c.Set_NProduccion(prod_le.Get_NProduccion() + 1);
    pila_producciones.push(prod_le_c);    
  }
  else if (simbolo_semantico == S_97_0) {
    
  }
  else if (simbolo_semantico == S_98_0) {
    
  }
  else if (simbolo_semantico == S_98_1) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);

    if (prod_e.Get_Tipo() != T_TD_BOOLEAN)
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);
  }
  else if (simbolo_semantico == S_98_2) {
    
  }
  else if (simbolo_semantico == S_98_3) {
    CProduccion prod_ds = Get_Primera_Produccion(NT_DO_STATMENT);
    CProduccion prod_s = gramatica[37];

    prod_s.Set_Clase(prod_ds.Get_Clase());
    prod_s.Set_Valor(prod_ds.Get_Valor());
    pila_producciones.push(prod_s);
  }
  else if (simbolo_semantico == S_99_0) {
    
  }
  else if (simbolo_semantico == S_99_1) {
    CProduccion prod_cd = Get_Primera_Produccion(NT_CONSTRUCTOR_DECLARATION);
    CProduccion prod_sb = gramatica[34];

    prod_sb.Set_Clase(prod_cd.Get_Clase());
    pila_producciones.push(prod_sb);    
  }
  else if (simbolo_semantico == S_99_2) {
    CProduccion prod_cd = Get_Primera_Produccion(NT_CONSTRUCTOR_DECLARATION);
    CProduccion prod_opl = gramatica[18];

    prod_opl.Set_Clase(prod_cd.Get_Clase());
    pila_producciones.push(prod_opl);      

    Tipo_Metodo metodo;
    metodo.acceso = A_PUBLIC;
    metodo.tipo = "";
    metodo.id = prod_cd.Get_Clase();

    // DEBUG ---------------
//    cout<<"**** "<<prod_cd.Get_Clase()<<" ****"<<endl;
    // DEBUG ---------------

    CLexema cl = tabla_simbolos->Recupera(prod_cd.Get_Clase());
    CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
    cld->Inserta_Metodo(metodo);
    cl.SetLexema(cl.GetID(), cl.GetTipo(), (void*)cld);
    tabla_simbolos->Modifica(cl);
  }
  else if (simbolo_semantico == S_100_0) {
    CProduccion prod_cl = Get_Primera_Produccion(NT_CONSTANTE_LOGICA);
    prod_cl.Set_Valor(T_TRUE);
    prod_cl.Set_Tipo(T_TD_BOOLEAN);
    Set_Primera_Produccion(prod_cl);
  }
  else if (simbolo_semantico == S_101_0) {
    CProduccion prod_cl = Get_Primera_Produccion(NT_CONSTANTE_LOGICA);
    prod_cl.Set_Valor(T_FALSE);
    prod_cl.Set_Tipo(T_TD_BOOLEAN);
    Set_Primera_Produccion(prod_cl);    
  }
  else if (simbolo_semantico == S_102_0) {
    
  }
  else if (simbolo_semantico == S_102_1) {
    
  }
  else if (simbolo_semantico == S_102_2) {

  }
  else if (simbolo_semantico == S_103_0) {
    
  }
  else if (simbolo_semantico == S_104_0) {
    
  }
  else if (simbolo_semantico == S_105_0) {
    
  }
  else if (simbolo_semantico == S_105_1) {
    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_ep = gramatica[55];

    CLexema lex_var = tabla_simbolos->Recupera(prod_id.Get_Valor());
    if (lex_var.GetTipo() != L_VARIABLE)
      throw Semantico_Error_Ex(NO_ES_UNA_VARIABLE);

    CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
    prod_ep.Set_Clase(vd->GetTipoDeDatos());

    prod_ep.Set_Valor(prod_ps.Get_Valor());
    pila_producciones.push(prod_ep);    
  }  
  else if (simbolo_semantico == S_106_0) {
    
  }
  else if (simbolo_semantico == S_106_1) {
    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);

    if (prod_ps.Get_Valor() != "read" && prod_ps.Get_Valor() != "print") {
      CLexema lex = tabla_simbolos->Recupera(prod_ps.Get_Clase());
      CClaseDatos* cld = (CClaseDatos*)lex.GetDatos();

      if (!cld->Es_Metodo(prod_ps.Get_Valor()))
 	throw Semantico_Error_Ex(FUNCION_NO_VISIBLE);
    }

    CProduccion prod_ae = gramatica[84];
    prod_ae.Set_Valor(prod_ps.Get_Valor());
    prod_ae.Set_Clase(prod_ps.Get_Clase());
    pila_producciones.push(prod_ae);
  }
  else if (simbolo_semantico == S_107_0) {
    CProduccion prod_e = Get_Primera_Produccion(NT_EXPRESION);
    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);

    // DEBUG -------------
//     cout<<"**** "<<prod_e.Get_Valor()<<":"<<prod_e.Get_Tipo()<<" ****"<<endl;
//     cout<<"**** "<<prod_ps.Get_Valor()<<":"<<prod_ps.Get_Tipo()<<" ****"<<endl;
    // DEBUG -------------
    if (prod_e.Get_Tipo() != prod_ps.Get_Tipo())
      throw Semantico_Error_Ex(VIOLACION_DE_TIPOS);
  }
  else if (simbolo_semantico == S_107_1) {
    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);
    CProduccion prod_e = gramatica[54];

    // DEBUG --------------
//    cout<<"**** "<<prod_ps.Get_Valor()<<" ****"<<endl;
    // DEBUG -------------
    CLexema lex = tabla_simbolos->Recupera(prod_ps.Get_Valor());
    if (lex.GetTipo() == L_CONSTANTE)
      throw Semantico_Error_Ex(NO_A_CONSTANTE);

    prod_e.Set_Valor(prod_ps.Get_Valor());
    prod_e.Set_Clase(prod_ps.Get_Clase());
    pila_producciones.push(prod_e);    
  }
  else if (simbolo_semantico == S_107_2) {
    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);
   
    if (!tabla_simbolos->Busca(prod_ps.Get_Valor()))
      throw Semantico_Error_Ex(VARIABLE_NO_DECLARADA);
  }
  else if (simbolo_semantico == S_108_0) {

  }
  else if (simbolo_semantico == S_108_1) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CProduccion prod_ps = Get_Primera_Produccion(NT_POSIBLE_STATMENT);
    CVariableDatos* vd = new CVariableDatos(prod_ps.Get_Tipo(), "");
    CLexema lex(prod_id.Get_Valor(), L_VARIABLE, (void*)vd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }

    CProduccion prod_vd = gramatica[47];
    prod_vd.Set_Tipo(prod_ps.Get_Tipo());
    pila_producciones.push(prod_vd);
  }
  else if (simbolo_semantico == S_109_0) {

  }
  else if (simbolo_semantico == S_110_0) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_il = Get_Primera_Produccion(NT_INTEGER_LITERAL);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CConstanteDatos* cd = new CConstanteDatos(T_TD_INT, prod_il.Get_Valor());
    CLexema lex(prod_id.Get_Valor(), L_CONSTANTE, (void*)cd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }
  }
  else if (simbolo_semantico == S_111_0) {
    CProduccion prod_id = Get_Primera_Produccion(NT_IDENTIFIER);
    CProduccion prod_cl = Get_Primera_Produccion(NT_CONSTANTE_LOGICA);
    if (tabla_simbolos->Busca(prod_id.Get_Valor()))
      throw Semantico_Error_Ex(REDEFINICION);

    CConstanteDatos* cd = new CConstanteDatos(T_TD_BOOLEAN, prod_cl.Get_Valor());
    CLexema lex(prod_id.Get_Valor(), L_CONSTANTE, (void*)cd);
    try {
      tabla_simbolos->Inserta(lex);
    }
    catch ( Palabra_Reservada_Ex ) {
      throw Semantico_Error_Ex(PALABRA_RESERVADA);
    }    
  }
  else if (simbolo_semantico == S_112_0) {

  }
}

void
CCompilador::Generacion_Codigo(const string& simbolo_gencodigo)
{
  pila_gramatical.pop();
  
  CProduccion      prod, prod1, prod2, prod3, prod_aux;    // Para coger producciones de la pila
  string           temp, temp2;                            // Para el nombre del temporal
  string           etiq, etiq2;                            // Para las etiquetas
  string           codigo;                                 // Para el cdigo generado
  vector<string>   vcodigo, vcodigo2;                      // Para los vectores de codigo
  vector<string>   vobjetos;                               // Para guardar posibles objetos
  string           objeto;

  char             Num[80];                                // Para transformar nmeros en cadenas


  // *************************************************************************
  // *** TEMPORAL. Para que genere cdigo cuando se queja el semntico *******
  //hubo_error = false;


  if (simbolo_gencodigo == G_1_0) { //compilation_unit ::= lista_type_declaration
    if (!hubo_error){
      prod = Get_Primera_Produccion ("compilation_unit");
      prod1 = Get_Primera_Produccion ("lista_type_declaration");
      
      //cout << "ANTES DE AADIR GOTO MAIN" << endl;
      prod.Add_Codigo ("goto MAIN");
      prod.Add_Codigo ("");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      
      Mostrar_Codigo(prod.Get_Codigo());
      
      Escribir_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(1);
  }
  else if (simbolo_gencodigo == G_2_0) { // lista_type_declaration ::= type_declaration lista_type_declaration
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_type_declaration");
      prod1 = Get_Primera_Produccion ("type_declaration");
      prod2 = Get_Primera_Produccion ("lista_type_declaration");

      prod.Set_Codigo (Concatenar_Vectores(prod1.Get_Codigo(), prod.Get_Codigo()));
      //Escribir_Codigo (prod.Get_Codigo());
      //prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      
      Mostrar_Codigo(prod.Get_Codigo());

      if (opciones->Debug()) { 
	cout << "PROD2.LUGAR: " << prod.Get_Lugar() << endl;
	Mostrar_Estado_Pila_Entrada(); 
	Mostrar_Pila_Producciones();
      }

      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(2);
  }
  else if (simbolo_gencodigo == G_3_0) { // lista_type_declaration ::= "landa"
    if (!hubo_error){
     }
    Desapilar_Produccion(3);
  }
  else if (simbolo_gencodigo == G_4_0) { // type_declaration ::= class_declaration
    if (!hubo_error){
      prod = Get_Primera_Produccion ("type_declaration");
      prod1 = Get_Primera_Produccion ("class_declaration");

      prod.Set_Codigo (prod1.Get_Codigo());

      Mostrar_Codigo(prod.Get_Codigo());
      
      if (opciones->Debug()) { 
	Mostrar_Estado_Pila_Entrada(); 
	Mostrar_Pila_Producciones();
      }
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(4);
  }
  else if (simbolo_gencodigo == G_5_0) { // class_declaration ::= modifier "class" identifier "{" lista_field_declaration "}"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("class_declaration");
      prod2 = Get_Primera_Produccion ("identifier");
      prod3 = Get_Primera_Produccion ("lista_field_declaration");

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));

      //      CLexema lex_var = tabla_simbolos->Recupera(prod2.Get_Lugar());
      //CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
      CLexema cl = tabla_simbolos->Recupera(prod2.Get_Lugar());
      //CLexema cl = tabla_simbolos->Recupera(prod1.Get_Lugar());
      if (opciones->Debug())
	cout << "CLASE ENCONTRADA: " << cl.GetID() << endl;
      CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
      list<Tipo_Metodo> metodos = (cld->Get_Metodos());
      list<Tipo_Metodo>::iterator it;
      it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(prod2.Get_Lugar()));
      if (it != metodos.end()){
	if (opciones->Debug()){
	  cout << "METODO ENCONTRADO: " << it->id;
	  cout << "TIPO_METODO: " << it->tipo;
	  cout << "ACCESO_METODO: " << it->acceso;
	}
      } else {
	if (opciones->Debug())
	  cout << "CREAR CONSTRUCTOR" << endl;
	prod.Add_Codigo(prod2.Get_Lugar() + ":");
	prod.Add_Codigo("RETURN");
	prod.Add_Codigo("");
	prod.Add_Codigo("");
      }
      

      Mostrar_Codigo(prod.Get_Codigo());

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(5);
  }
  else if (simbolo_gencodigo == G_6_0) { // lista_field_declaration ::= field_declaration lista_field_declaration
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_field_declaration");
      prod1 = Get_Primera_Produccion ("field_declaration");
      prod2 = Get_Primera_Produccion ("lista_field_declaration");

      prod.Set_Codigo (Concatenar_Vectores (prod1.Get_Codigo(), prod.Get_Codigo()));
      //prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));

      Mostrar_Codigo(prod.Get_Codigo());

      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(6);
  }
  else if (simbolo_gencodigo == G_7_0) { // lista_field_declaration ::= "landa"
    if (!hubo_error){
    }
    Desapilar_Produccion(7);
  }
  else if (simbolo_gencodigo == G_8_0) { // modifier ::= "public"
    if (!hubo_error){
    }
    Desapilar_Produccion(8);
  }
  else if (simbolo_gencodigo == G_9_0) { // modifier ::= "private"
    if (!hubo_error){
    }
    Desapilar_Produccion(9);
  }
  else if (simbolo_gencodigo == G_10_0) { //  modifier ::= "landa"
    if (!hubo_error){
    }
    Desapilar_Produccion(10);
  }
  else if (simbolo_gencodigo == G_11_0) { // identifier ::= "identifier"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("identifier");
      prod.Set_Lugar((string) analizador_lexico->YYText());
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(11);
  }
  else if (simbolo_gencodigo == G_12_0) { // field_declaration ::= modifier type field_declaration_prima
    if (!hubo_error){
      prod = Get_Primera_Produccion ("field_declaration");
      prod1 = Get_Primera_Produccion ("modifier");
      prod2 = Get_Primera_Produccion ("type");
      prod3 = Get_Primera_Produccion ("field_declaration_prima");

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));
      
      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }

      // Escribir_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(12);
  }
  else if (simbolo_gencodigo == G_13_0) { // field_declaration ::= ";"
    if (!hubo_error){
    }
    Desapilar_Produccion(13);
  }
  else if (simbolo_gencodigo == G_14_0) { // field_declaration_prima := constructor_declaration 
    if (!hubo_error){
      prod = Get_Primera_Produccion ("field_declaration_prima");
      prod1 = Get_Primera_Produccion ("constructor_declaration");
      
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      
      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }
      
      Mostrar_Codigo (prod1.Get_Codigo());
      Mostrar_Codigo (prod.Get_Codigo());
      


      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(14);
  }
  else if (simbolo_gencodigo == G_15_0) { // field_declaration_prima := identifier field_declaration_prima_prima
    if (!hubo_error){
      prod = Get_Primera_Produccion ("field_declaration_prima");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("field_declaration_prima_prima");
      
      
      if (prod2.Get_Lugar() == "method"){
	if (prod1.Get_Lugar() == "main")
	  prod.Add_Codigo ("MAIN:");
	else{
	  prod_aux = Get_Primera_Produccion ("lista_field_declaration");
	  CLexema cl = tabla_simbolos->Recupera(prod_aux.Get_Clase());
	  if (opciones->Debug())
	    cout << "CLASE ENCONTRADA: " << cl.GetID() << endl;
	  prod.Add_Codigo (cl.GetID() + prod1.Get_Lugar() + ":");
	}
      }
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Set_Lugar (prod1.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(15);
  }
  else if (simbolo_gencodigo == G_16_0) { // field_declaration_prima_prima := variable_declaration
    if (!hubo_error){
      prod.Set_Lugar ("variable");
    }
    Desapilar_Produccion(16);
  }
  else if (simbolo_gencodigo == G_17_0) { //  field_declaration_prima_prima := method_declaration
    if (!hubo_error){
      prod = Get_Primera_Produccion ("field_declaration_prima_prima");
      prod1 = Get_Primera_Produccion ("method_declaration");
      
      prod.Set_Lugar ("method");
      prod.Set_Codigo (prod1.Get_Codigo());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(17);
  }
  else if (simbolo_gencodigo == G_18_0) { //  method_declaration ::= "(" opcional_parameter_list ")" lista_corchetes method_declaration_final
    if (!hubo_error){
      prod = Get_Primera_Produccion ("method_declaration");
      prod1 = Get_Primera_Produccion ("opcional_parameter_list");
      prod2 = Get_Primera_Produccion ("lista_corchetes");
      prod3 = Get_Primera_Produccion ("method_declaration_final");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_Codigo()));
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));
      prod.Add_Codigo ("RETURN");
      prod.Add_Codigo ("");
      prod.Add_Codigo ("");
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(18);
  }
  else if (simbolo_gencodigo == G_19_0) { // opcional_parameter_list ::= parameter_list
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_parameter_list");
      prod1 = Get_Primera_Produccion ("parameter_list");
      prod.Set_Codigo (prod1.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(19);
  }
  else if (simbolo_gencodigo == G_20_0) { // opcional_parameter_list ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_parameter_list");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(20);
  }
  else if (simbolo_gencodigo == G_21_0) { // lista_corchetes ::= "[" "]" lista_corchetes
    if (!hubo_error){
    }
    Desapilar_Produccion(21);
  }
  else if (simbolo_gencodigo == G_22_0) { // lista_corchetes ::= "landa"
    if (!hubo_error){
    }
    Desapilar_Produccion(22);
  }
  else if (simbolo_gencodigo == G_23_0) { // method_declaration_final ::= statment_block
    if (!hubo_error){
      prod = Get_Primera_Produccion ("method_declaration_final");
      prod1 = Get_Primera_Produccion ("statment_block");
      
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(23);
  }
  else if (simbolo_gencodigo == G_24_0) { //  method_declaration_final ::= ";"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("method_declaration_final");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(24);
  }
  else if (simbolo_gencodigo == G_25_0) { // type ::= type_specifier lista_corchetes
    if (!hubo_error){
    }
    Desapilar_Produccion(25);
  }
  else if (simbolo_gencodigo == G_26_0) { // type_specifier ::= "void" 
    if (!hubo_error){
    }
    Desapilar_Produccion(26);
  }
  else if (simbolo_gencodigo == G_27_0) { // type_specifier ::= "boolean"
    if (!hubo_error){
    }
    Desapilar_Produccion(27);
  }
  else if (simbolo_gencodigo == G_28_0) { // type_specifier ::= "int"
    if (!hubo_error){
    }
    Desapilar_Produccion(28);
  }
  else if (simbolo_gencodigo == G_29_0) { // type_specifier ::= class_name
    if (!hubo_error){
    }
    Desapilar_Produccion(29);
  }
  else if (simbolo_gencodigo == G_30_0) { // class_name ::= identifier
    if (!hubo_error){
    }
    Desapilar_Produccion(30);
  }
  else if (simbolo_gencodigo == G_31_0) { // parameter_list ::= parameter lista_parameter_list
    if (!hubo_error){
      prod = Get_Primera_Produccion ("parameter_list");
      prod1 = Get_Primera_Produccion ("parameter");
      prod2 = Get_Primera_Produccion ("lista_parameter_list");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo ("pop " + prod1.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(31);
  }
  else if (simbolo_gencodigo == G_32_0) { // lista_parameter_list ::= "," parameter lista_parameter_list
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_parameter_list");
      prod1 = Get_Primera_Produccion ("parameter");
      prod2 = Get_Primera_Produccion ("lista_parameter_list");
      
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo ("pop " + prod1.Get_Lugar());
      
      Mostrar_Codigo( prod.Get_Codigo());
      
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(32);
  }
  else if (simbolo_gencodigo == G_33_0) { // lista_parameter_list ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("lista_parameter_list");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(33);
  }
  else if (simbolo_gencodigo == G_34_0) { // parameter ::= type identifier lista_corchetes
    if (!hubo_error){
      prod = Get_Primera_Produccion ("parameter");
      prod1 = Get_Primera_Produccion ("type");
      prod2 = Get_Primera_Produccion ("identifier");
      prod3 = Get_Primera_Produccion ("lista_corchetes");
      
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar (prod2.Get_Lugar());
      
      if (opciones->Debug())
	cout << "LUGAR: " << prod.Get_Lugar() << endl;
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(34);
  }
  else if (simbolo_gencodigo == G_35_0) { // statement_block ::= "{" lista_statment "}"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment_block");
      prod1 = Get_Primera_Produccion ("lista_statment");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(35);
  }
  else if (simbolo_gencodigo == G_36_0) { // lista_statment ::= statment lista_statment
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_statment");
      prod1 = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("lista_statment");
      prod.Set_Lugar ("");
      prod.Set_Codigo (Concatenar_Vectores (prod1.Get_Codigo(), prod.Get_Codigo()));
      //prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(36);
  }
  else if (simbolo_gencodigo == G_37_0) { // lista_statment ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("lista_statment");
      prod.Set_Lugar ("");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(37);
  }
  else if (simbolo_gencodigo == G_38_0) { // statment ::= identifier posible_statment;
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("posible_statment");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      if (prod2.Get_TProd() == "asig")
	prod.Add_Codigo (prod1.Get_Lugar() + prod2.Get_Lugar());
      else if (prod2.Get_TProd() == "agrupa"){
	// Compruebo si el mtodo devuelve algn valor, para sacarlo de la pila 
	if (prod1.Get_Lugar() == "print")
	  prod.Add_Codigo ("OUTPUTCHAR " + prod2.Get_Lugar());
	else{

	  prod_aux = Get_Primera_Produccion ("lista_field_declaration");
	  CLexema cl = tabla_simbolos->Recupera(prod_aux.Get_Clase());
	  prod.Add_Codigo ("call " + cl.GetID() + prod1.Get_Lugar());
	  
	  if (opciones->Debug())
	    cout << "CLASE ENCONTRADA: " << cl.GetID() << endl;
	  CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
	  list<Tipo_Metodo> metodos = (cld->Get_Metodos());
	  list<Tipo_Metodo>::iterator it;
	  it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(prod1.Get_Lugar()));
	  if (it != metodos.end()){
	    if (opciones->Debug()){
	      cout << "METODO ENCONTRADO: " << it->id;
	      cout << "TIPO_METODO: " << it->tipo;
	      cout << "ACCESO_METODO: " << it->acceso;
	    }
	    if (it->tipo == "int" || it->tipo == "boolean"){
	      temp = Nuevo_Temporal();
	      prod.Add_Codigo ("pop " + temp);
	      prod.Set_Lugar (temp);
	    }
	  }
	}
      } else 
	if (prod2.Get_TProd() == "punto"){ // Punto ....
	  
	  //prod.Add_Codigo ("call " + prod1.Get_Lugar() + prod2.Get_Lugar());
	  
	  if (opciones->Debug())
	    cout << "PUNTITO: " << endl;
	  Mostrar_Codigo (prod.Get_Codigo());
	  
	  //vcodigo.clear();
	  //if (vcodigo != prod.Get_Codigo()){
	  //  vcodigo = prod.Get_Codigo();
	  //  vcodigo.pop_back();
	  //  vcodigo.pop_back();
	  //  prod.Set_Codigo (vcodigo);
	  //}

	  // Compruebo si el mtodo devuelve algn valor, para sacarlo de la pila 
   	  CLexema lex_var = tabla_simbolos->Recupera(prod1.Get_Lugar());
	  CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
	  CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
	  //CLexema cl = tabla_simbolos->Recupera(prod1.Get_Lugar());
	  if (opciones->Debug())
	    cout << "CLASE ENCONTRADA: " << cl.GetID() << endl;
	  prod.Add_Codigo ("call " + cl.GetID() + prod2.Get_Lugar());
	  CClaseDatos* cld = (CClaseDatos*)cl.GetDatos();
	  list<Tipo_Metodo> metodos = (cld->Get_Metodos());
	  list<Tipo_Metodo>::iterator it;
	  it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(prod2.Get_Lugar()));
	  if (it != metodos.end()){
	    if (opciones->Debug()){
	      cout << "METODO ENCONTRADO: " << it->id;
	      cout << "TIPO_METODO: " << it->tipo;
	      cout << "ACCESO_METODO: " << it->acceso;
	    }
	    if (it->tipo == "int" || it->tipo == "boolean"){
	      temp = Nuevo_Temporal();
	      prod.Add_Codigo ("pop " + temp);
	      prod.Set_Lugar (temp);
	    }
	  }
	}
	else if (prod2.Get_TProd() == "obj") {
	  
	  if (prod1.Get_Lugar() != "int" && prod1.Get_Lugar() !="boolean"){
	    vobjetos = prod2.Get_LObjetos();
	    while (!vobjetos.empty()){
	      objeto = vobjetos[vobjetos.size()-1];
	      prod.Add_Codigo ("call " + prod1.Get_Lugar());
	      vobjetos.pop_back();
	    }
	    
	  }
	  
	  
	}
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_PCodigo()));

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(38);
  }
  else if (simbolo_gencodigo == G_39_0) { // statment ::= "void" identifier variable_declaration; **************
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("identifier");
      prod3 = Get_Primera_Produccion ("variable_declaration");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));
      if (prod3.Get_Lugar() != "")
	prod.Add_Codigo (prod2.Get_Lugar() + prod3.Get_Lugar());
      
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(39);
  }
  else if (simbolo_gencodigo == G_40_0) { // statment ::= "int" identifier variable_declaration;
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("identifier");
      prod3 = Get_Primera_Produccion ("variable_declaration");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));
      if (prod3.Get_Lugar() != "")
	prod.Add_Codigo (prod2.Get_Lugar() + prod3.Get_Lugar());
      
      
      Mostrar_Codigo (prod.Get_Codigo());

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(40);
  }
  else if (simbolo_gencodigo == G_41_0) { // statment ::= "boolean" identifier variable_declaration;
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("identifier");
      prod3 = Get_Primera_Produccion ("variable_declaration");

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod3.Get_Codigo()));
      if (prod3.Get_Lugar() != "")
	prod.Add_Codigo (prod2.Get_Lugar() + prod3.Get_Lugar());
      
      
      Mostrar_Codigo (prod.Get_Codigo());

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(41);
  }
  else if (simbolo_gencodigo == G_42_0) { // statment ::= statment_block
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod1 = Get_Primera_Produccion ("statment_block");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
            
      Mostrar_Codigo (prod.Get_Codigo());

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(42);
  }
  else if (simbolo_gencodigo == G_43_0) { // statment ::= do_statment
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod1 = Get_Primera_Produccion ("do_statment");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_PCodigo()));
      
      //Escribir_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(43);
  }
  else if (simbolo_gencodigo == G_44_0) { // statment ::= "return" opcional_expresion ";"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("opcional_expresion");
      
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      if (prod2.Get_Lugar() != "") 
	prod.Add_Codigo ("push " + prod2.Get_Lugar());

      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_PCodigo()));
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(44);
  }
  else if (simbolo_gencodigo == G_45_0) { // statment ::= ";"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(45);
  }
  else if (simbolo_gencodigo == G_46_0) { // opcional_expresion ::= expresion 
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_expresion");
      prod1 = Get_Primera_Produccion ("expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(46);
  }
  else if (simbolo_gencodigo == G_47_0) { // opcional_expresion := "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_expresion");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(47);
  }
  else if (simbolo_gencodigo == G_48_0) { // variable_declaration ::= variable_declarator lista_variable_declarator ";"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("variable_declaration");
      prod1 = Get_Primera_Produccion ("variable_declarator");
      prod2 = Get_Primera_Produccion ("lista_variable_declarator");
      
      prod.Set_Lugar (prod1.Get_Lugar());

      if (opciones->Debug())
	cout << "TIPOVARIABLEDECLARATION" << prod1.Get_Valor()<< endl;

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_Codigo()));
      if (prod1.Get_TProd() == "obj"){
	prod_aux = Get_Primera_Produccion ("identifier");
	CLexema lex_var = tabla_simbolos->Recupera(prod_aux.Get_Lugar());
	CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
	CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
	
	prod.Add_Codigo ("call " + cl.GetID());
      }
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));

      // Pueden ser objetos. Para hacer despues la generacin de cdigo.
      prod.Set_LObjetos (prod2.Get_LObjetos());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(48);
  }
  else if (simbolo_gencodigo == G_49_0) { // lista_variable_declarator ::= "," identifier variable_declarator lista_variable_declarator
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_variable_declarator");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("variable_declarator");
      prod3 = Get_Primera_Produccion ("lista_variable_declarator");

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      if (prod2.Get_TProd() == "obj"){
	CLexema lex_var = tabla_simbolos->Recupera(prod1.Get_Lugar());
	CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
	CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());

	prod.Add_Codigo ("call " + cl.GetID());
      } else if (prod2.Get_TProd() == "asig")
	prod.Add_Codigo (prod1.Get_Lugar() + prod2.Get_Lugar());
      else //if (prod2.Get_Lugar() == "")
	prod.Add_Objeto (prod1.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());

      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(49);
  }
  else if (simbolo_gencodigo == G_50_0) { // lista_variable_declarator ::= "landa"
    if (!hubo_error){
    }
    Desapilar_Produccion(50);
  }
  else if (simbolo_gencodigo == G_51_0) { // variable_declarator ::= lista_corchetes opcional_variable_initializer
    if (!hubo_error){
      prod = Get_Primera_Produccion ("variable_declarator");
      prod2 = Get_Primera_Produccion ("opcional_variable_initializer");
      prod.Set_Lugar (prod2.Get_Lugar());
      prod.Set_Codigo (prod2.Get_Codigo());

      Mostrar_Codigo (prod.Get_Codigo());
      
      prod.Set_TProd(prod2.Get_TProd());
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(51);
  }
  else if (simbolo_gencodigo == G_52_0) { // opcional_variable_initializer ::= "=" variable_initializer
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_variable_initializer");
      prod1 = Get_Primera_Produccion ("variable_initializer");
      prod.Set_Lugar ("= " + prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      Mostrar_Codigo (prod.Get_Codigo());

      prod.Set_TProd("asig");

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(52);
  }
  else if (simbolo_gencodigo == G_53_0) { // opcional_variable_initializer ::= "landa"
    if (!hubo_error){
      
    }
    Desapilar_Produccion(53);
  }
  else if (simbolo_gencodigo == G_54_0) { // variable_initializer ::= expresion
    if (!hubo_error){
      prod = Get_Primera_Produccion ("variable_initializer");
      prod1 = Get_Primera_Produccion ("expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      Mostrar_Codigo (prod.Get_Codigo());

      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(54);
  }
  else if (simbolo_gencodigo == G_55_0) { // expresion ::= expresion_1 expresion_prima 
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion");
      prod1 = Get_Primera_Produccion ("expresion_1");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      prod2 = Get_Primera_Produccion ("expresion_prima");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      
      prod.Set_Lugar (prod1.Get_Lugar());
      if (opciones->Debug())
	cout << "PROD2.GET_LUGAR =" << prod2.Get_Lugar() << ";" << endl;
      if (prod2.Get_Lugar() != "" )
	if (prod2.Get_TProd() == "inde"){ // Indecremento
	  temp2 = Nuevo_Temporal ();
	  prod.Add_PCodigo (temp2 + " =  1");
	  //temp = Nuevo_Temporal ();
	  if (prod2.Get_Lugar() == " - 1")
	    prod.Add_PCodigo (prod1.Get_Lugar() + " = " + prod1.Get_Lugar() + " - " + temp2);
	  else
	    prod.Add_PCodigo (prod1.Get_Lugar() + " = " + prod1.Get_Lugar() + " + " + temp2);
	  prod.Set_Lugar (prod1.Get_Lugar());
	} else
	  if (prod2.Get_TProd() == "punto"){ // Punto ....
	    //vcodigo = prod.Get_Codigo();
	    //vcodigo.pop_back();
	    //vcodigo.pop_back();
	    //prod.Set_Codigo (vcodigo);
	    vobjetos = prod2.Get_LObjetos();
	    prod_aux = Get_Segunda_Produccion ("expresion_1");
	    CLexema lex_var = tabla_simbolos->Recupera(prod_aux.Get_Lugar());
	    CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
	    CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
	    prod.Add_Codigo ("call " + cl.GetID() + vobjetos[vobjetos.size()-1]);
	    temp = Nuevo_Temporal();
	    prod.Add_Codigo ("pop " + temp);
	    prod.Set_Lugar(temp);
	    //prod.Add_Codigo ("llllcall " + prod1.Get_Lugar() + prod2.Get_Lugar());
	    //temp = Nuevo_Temporal();
	    //prod.Add_Codigo ("pop " + temp);
	    //prod.Set_Lugar (temp);
	  } else{
	    if (prod1.Get_TProd() == "agrupa" || prod2.Get_TProd() == "agrupa") {
	      prod.Set_Lugar(prod2.Get_Lugar());
	    }else {
	      if (opciones->Debug())
		cout << "entra en el if" << endl;
	      temp = Nuevo_Temporal ();
	      prod.Add_Codigo (temp + " = " + prod1.Get_Lugar() + prod2.Get_Lugar());
	      prod.Set_Lugar(temp);
	    }
	  }
      if (prod1.Get_TProd() == "agrupa") {
	      if (opciones->Debug()) cout << "ES UNA FUNCION" << endl;
	      prod.Set_Lugar(prod1.Get_Lugar());
	    }
      //    else
	      //prod.Add_Codigo (prod1.Get_Lugar() + prod2.Get_Lugar());
      //      prod.Set_Lugar (prod1.Get_Lugar() + prod2.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(55);
  }
  else if (simbolo_gencodigo == G_56_0) { // expresion_prima ::= op_indecremento expresion_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_prima");
      prod1 = Get_Primera_Produccion ("op_indecremento");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      prod2 = Get_Primera_Produccion ("expresion_prima");
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));

      if (prod2.Get_Lugar() != ""){
	temp = Nuevo_Temporal();
	prod.Add_PCodigo (temp + " = " + prod2.Get_Lugar()); 
	if (prod1.Get_Lugar() == " ++ ")
	  prod.Set_Lugar (temp + " + 1");
	else
	  prod.Set_Lugar (temp + " - 1");
      } else
	if (prod1.Get_Lugar() == " ++ ")
	  prod.Set_Lugar (prod.Get_Lugar() + " + 1");
	else
	  prod.Set_Lugar (prod.Get_Lugar() + " - 1");
      
      prod.Set_TProd("inde");

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(56);
  }
  else if (simbolo_gencodigo == G_57_0) { // expresion_prima ::= op_aritmetico expresion_1 expresion_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_prima");
      prod1 = Get_Primera_Produccion ("op_aritmetico");
      prod2 = Get_Primera_Produccion ("expresion_1");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      prod3 = Get_Primera_Produccion ("expresion_prima");
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod3.Get_PCodigo()));

      if (prod3.Get_Lugar() != ""){
	if (prod3.Get_TProd() == "punto"){ // Punto ....
	  //vcodigo = prod.Get_Codigo();
	  //vcodigo.pop_back();
	 //  vcodigo.pop_back();
// 	  prod.Set_Codigo (vcodigo);
// 	  vobjetos = prod2.Get_LObjetos();
// 	  prod_aux = Get_Segunda_Produccion ("expresion_1");
// 	  CLexema lex_var = tabla_simbolos->Recupera(prod_aux.Get_Lugar());
// 	  CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
// 	  CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
// 	  prod.Add_Codigo ("CALL " + cl.GetID() + vobjetos[vobjetos.size()-1]);
// 	  temp = Nuevo_Temporal();
// 	  prod.Add_Codigo ("pop " + temp);
// 	  prod.Set_Lugar(temp);
	  //prod.Add_Codigo ("llllcall " + prod1.Get_Lugar() + prod2.Get_Lugar());
	  //temp = Nuevo_Temporal();
	  //prod.Add_Codigo ("pop " + temp);
	  //prod.Set_Lugar (temp);
	  
	  prod.Set_TProd("");
	  
	}
	temp = Nuevo_Temporal();
	prod.Add_Codigo (temp + " = " + prod2.Get_Lugar() + prod3.Get_Lugar()); 
	prod.Set_Lugar (prod1.Get_Lugar() + temp);
	
      } else
	prod.Set_Lugar (prod.Get_Lugar() + prod1.Get_Lugar() + prod2.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
        if (opciones->Debug()) { 
	    Mostrar_Estado_Pila_Entrada(); 
	    Mostrar_Pila_Producciones();
	  }
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(57);
  }
  else if (simbolo_gencodigo == G_58_0) { // expresion_prima ::= op_relacional expresion_1 expresion_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_prima");
      prod1 = Get_Primera_Produccion ("op_relacional");
      prod2 = Get_Primera_Produccion ("expresion_1");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      prod3 = Get_Primera_Produccion ("expresion_prima");
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod3.Get_PCodigo()));

      if (prod3.Get_Lugar() != ""){
	temp = Nuevo_Temporal();
	prod.Add_Codigo (temp + " = " + prod2.Get_Lugar() + prod3.Get_Lugar()); 
	prod.Set_Lugar (prod1.Get_Lugar() + temp);
      } else
	prod.Set_Lugar (prod.Get_Lugar() + prod1.Get_Lugar() + prod2.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
        
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(58);
  }
  else if (simbolo_gencodigo == G_59_0) { // expresion_prima ::= op_logico expresion_1 expresion_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_prima");
      prod1 = Get_Primera_Produccion ("op_logico");
      prod2 = Get_Primera_Produccion ("expresion_1");
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      prod3 = Get_Primera_Produccion ("expresion_prima");
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod3.Get_PCodigo()));

      if (prod3.Get_Lugar() != ""){
	temp = Nuevo_Temporal();
	prod.Add_Codigo (temp + " = " + prod2.Get_Lugar() + prod3.Get_Lugar()); 
	prod.Set_Lugar (prod1.Get_Lugar() + temp);
      } else
	prod.Set_Lugar (prod.Get_Lugar() + prod1.Get_Lugar() + prod2.Get_Lugar());
      
      Mostrar_Codigo (prod.Get_Codigo());
        
      Set_Segunda_Produccion (prod);
      
    }
    Desapilar_Produccion(59);
  }
  else if (simbolo_gencodigo == G_60_0) { // expresion_prima ::= "." expresion_1 expresion_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_prima");
      prod1 = Get_Primera_Produccion ("expresion_1");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      
      

      prod2 = Get_Primera_Produccion ("expresion_prima");
      //prod.Set_Lugar (prod.Get_Lugar() + prod2.Get_Lugar());
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));

      /*if (prod1.Get_TProd() == "agrupa"){ // Punto ....
 	vcodigo = prod.Get_Codigo();
 	vcodigo.pop_back();
 	vcodigo.pop_back();
 	prod.Set_Codigo (vcodigo);
 	vobjetos = prod2.Get_LObjetos();
 	cout << "ANTES DE SEGUNDA PRODUCCION EXP_1" << endl;
 	prod_aux = Get_Segunda_Produccion ("expresion_1");
 	cout << "DESPUES DE SEGUNDA PRODUCCION EXP_1" << prod2.Get_Valor() <<endl;
 	CLexema lex_var = tabla_simbolos->Recupera(prod2.Get_Valor());
 	CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
 	CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
 	prod.Add_Codigo ("CALL " + cl.GetID() + prod1.Get_Lugar());
 	temp = Nuevo_Temporal();
 	prod.Add_Codigo ("pop " + temp);
 	prod.Set_Lugar(temp);
 	//prod.Add_Codigo ("llllcall " + prod1.Get_Lugar() + prod2.Get_Lugar());
 	//temp = Nuevo_Temporal();
 	//prod.Add_Codigo ("pop " + temp);
 	//prod.Set_Lugar (temp);
	}*/
    if (opciones->Debug()) {
 	Mostrar_Estado_Pila_Entrada();
 	Mostrar_Pila_Producciones();
      }
      prod.Set_LObjetos (prod1.Get_LObjetos());

      if (prod1.Get_TProd() != "agrupa")
	prod.Set_TProd ("punto");
      else prod.Set_TProd ("agrupa");

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(60);
  }
  else if (simbolo_gencodigo == G_61_0) { // expresion_prima ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_prima");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");

      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(61);
  }
  
  else if (simbolo_gencodigo == G_62_0) { // expresion_1 ::= expresion_2 expresion_1_prima
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_1");
      prod1 = Get_Primera_Produccion ("expresion_2");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      prod2 = Get_Primera_Produccion ("expresion_1_prima");
      //prod.Set_Lugar (prod.Get_Lugar() + prod2.Get_Lugar());
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      
      //prod.Set_TProd (prod2.Get_TProd());
      
      if (prod2.Get_TProd() == "agrupa") {  
	prod.Set_Lugar(prod1.Get_Lugar());
	
	prod.Set_TProd("agrupa");
	
	
	prod.Add_Codigo ("call " + prod1.Get_Clase() + prod1.Get_Lugar());
	temp = Nuevo_Temporal();
	prod.Add_Codigo ("pop " + temp);
	prod.Set_Lugar (temp);
      } 
      // Para guardar el nombre la funcin
      prod.Add_Objeto (prod1.Get_Lugar());
 
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(62);
  }
  else if (simbolo_gencodigo == G_63_0) { // expresion_1_prima ::= expresion_3 expresion_1_prima
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("expresion_1_prima");
      prod1 = Get_Primera_Produccion ("expresion_3");
      prod.Set_Lugar (prod1.Get_Lugar());    
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod1.Get_Codigo()));
      prod2 = Get_Primera_Produccion ("expresion_1_prima");
      prod.Set_Lugar (prod.Get_Lugar() + prod2.Get_Lugar());
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));

      prod.Set_TProd( "agrupa");
      
      Mostrar_Codigo (prod.Get_Codigo());
      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }
      Set_Segunda_Produccion (prod);
      if (opciones->Debug())
	cout << "DESPUES DE SET SEGUNDA PRODUCCION" << endl;
    }
    Desapilar_Produccion(63);
  }
  else if (simbolo_gencodigo == G_64_0) { // expresion_1_prima ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_1_prima");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");

      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(64);
  }
  else if (simbolo_gencodigo == G_65_0) { // expresion_2 ::= constante_logica
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_2");
      prod1 = Get_Primera_Produccion ("constante_logica");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(65);
  }
  else if (simbolo_gencodigo == G_66_0) { // expresion_2 ::= literal_expresion
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_2");
      prod1 = Get_Primera_Produccion ("literal_expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(66);
  }
  else if (simbolo_gencodigo == G_67_0) { // expresion_2 ::= identifier
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_2");
      prod1 = Get_Primera_Produccion ("identifier");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Set_Primera_Produccion(prod);
      if (opciones->Debug())
	cout <<"*****************"<<prod.Get_Lugar()<<"********"<<endl;
    }
    Desapilar_Produccion(67);
  }
  else if (simbolo_gencodigo == G_68_0) { // expresion_2 ::= "(" expresion ")"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_2");
      prod1 = Get_Primera_Produccion ("expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      // Nuevo
      //temp = Nuevo_Temporal();
      //prod.Add_Codigo (temp + " = " + prod1.Get_Lugar());
      //prod.Set_Lugar(temp);

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Set_Primera_Produccion(prod);
      if (opciones->Debug())
	cout <<"*****************"<<prod.Get_Lugar()<<"********"<<endl;
    }
    Desapilar_Produccion(68);
  }
  else if (simbolo_gencodigo == G_69_0) { // expresion_3 ::= agrupa_expresion
    if (!hubo_error){
      prod = Get_Primera_Produccion ("expresion_3");
      prod1 = Get_Primera_Produccion ("agrupa_expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo())); 

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(69);
  }
  else if (simbolo_gencodigo == G_70_0) { // op_indecremento ::= "++"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_indecremento");
      prod.Set_Lugar(" ++ ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(70);
  }
  else if (simbolo_gencodigo == G_71_0) { // op_indecremento ::= "--"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_indecremento");
      prod.Set_Lugar(" -- ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(71);
  }
  else if (simbolo_gencodigo == G_72_0) { // op_aritmetico ::= "+"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_aritmetico");
      prod.Set_Lugar(" + ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(72);
  }
  else if (simbolo_gencodigo == G_73_0) { // op_aritmetico ::= "-"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_aritmetico");
      prod.Set_Lugar(" - ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(73);
  }
  else if (simbolo_gencodigo == G_74_0) { // op_aritmetico ::= "*"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_aritmetico");
      prod.Set_Lugar(" * ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(74);
  }
  else if (simbolo_gencodigo == G_75_0) { // op_aritmetico ::= "/"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_aritmetico");
      prod.Set_Lugar(" / ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(75);
  }
  else if (simbolo_gencodigo == G_76_0) { // op_aritmetico ::= "%"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_aritmetico");
      prod.Set_Lugar(" % ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(76);
  }
  else if (simbolo_gencodigo == G_77_0) { // op_relacional ::= ">"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" > ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(77);
  }
  else if (simbolo_gencodigo == G_78_0) { // op_relacional ::= "<"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" < ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(78);
  }
  else if (simbolo_gencodigo == G_79_0) { // op_relacional ::= ">="
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" >= ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(79);
  }
  else if (simbolo_gencodigo == G_80_0) { // op_relacional ::= "<="
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" <= ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(80);
  }
  else if (simbolo_gencodigo == G_81_0) { // op_relacional ::= "=="
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" == ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(81);
  }
  else if (simbolo_gencodigo == G_82_0) { // op_relacional ::= "!=" 
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_relacional");
      prod.Set_Lugar(" != ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(82);
  }
  else if (simbolo_gencodigo == G_83_0) { // op_logico ::= "||"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_logico");
      prod.Set_Lugar(" || ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(83);
  }
  else if (simbolo_gencodigo == G_84_0) { // op_logico ::= "&&"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_logico");
      prod.Set_Lugar(" && ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(84);
  }
  else if (simbolo_gencodigo == G_85_0) { // agrupa_expresion ::= "(" opcional_arglist ")"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("agrupa_expresion");
      prod1 = Get_Primera_Produccion ("opcional_arglist");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(85);
  }
  else if (simbolo_gencodigo == G_86_0) { // agrupa_expresion ::= "[" expresion "]"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("agrupa_expresion");
      prod1 = Get_Primera_Produccion ("expresion");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(86);
  }
  else if (simbolo_gencodigo == G_87_0) { // opcional_arglist ::= arglist
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_arglist");
      prod1 = Get_Primera_Produccion ("arglist");
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(87);
  }
  else if (simbolo_gencodigo == G_88_0) { // opcional_arglist ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("opcional_arglist");
      vcodigo.clear();
      prod.Set_Codigo (vcodigo);
      prod.Set_Lugar ("");
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(88);
  }
  else if (simbolo_gencodigo == G_89_0) { // literal_expresion ::= integer_literal
    if (!hubo_error){
      prod = Get_Primera_Produccion ("literal_expresion");
      prod1 = Get_Primera_Produccion ("integer_literal");
      prod.Set_Lugar(prod1.Get_Lugar());
      prod.Set_Codigo(prod1.Get_Codigo());
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(89);
  }
  else if (simbolo_gencodigo == G_90_0) { // literal_expresion ::= string
    if (!hubo_error){
      prod = Get_Primera_Produccion ("literal_expresion");
      prod1 = Get_Primera_Produccion ("string");
      prod.Set_Lugar(prod1.Get_Lugar());
      prod.Set_Codigo(prod1.Get_Codigo());
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(90);
  }
  else if (simbolo_gencodigo == G_91_0) { // integer_literal ::= "octal"
    if (!hubo_error){
      // ************************************************************************
      // ********** FALTA PASAR OCTAL A DECIMAL    ******************************
      // ************************************************************************
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("integer_literal");
      prod.Set_Lugar(temp);
      sprintf (Num, "%d", atoi((analizador_lexico->YYText())));
      codigo = temp + " = " + (string) Num;
      prod.Add_Codigo(codigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(91);
  }
  else if (simbolo_gencodigo == G_92_0) { // integer_literal ::= "decimal"
    if (!hubo_error){
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("integer_literal");
      prod.Set_Lugar(temp);
      //codigo = temp + " = " + (string) analizador_lexico->YYText();
      // **************************
      codigo = temp + " = " + (string) analizador_lexico->YYText();
      prod.Add_Codigo(codigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(92);
  }
  else if (simbolo_gencodigo == G_93_0) { // integer_literal ::= "hexadecimal"
    if (!hubo_error){
      // ************************************************************************
      // ********** FALTA PASAR OCTAL A HEXADECIMAL    **************************
      // ************************************************************************
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("integer_literal");
      prod.Set_Lugar(temp);
      sprintf (Num, "%d", atoi((analizador_lexico->YYText())));
      codigo = temp + " = " + (string) Num;
      prod.Add_Codigo(codigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(93);
  }
  else if (simbolo_gencodigo == G_94_0) { // string ::= "cadena_caracteres"
    if (!hubo_error){
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("string");
      prod.Set_Lugar (temp);
      codigo = temp + " = " + (string) analizador_lexico->YYText();
      prod.Add_Codigo(codigo);
    
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(94);
  }
  else if (simbolo_gencodigo == G_95_0) { // arglist ::= expresion lista_expresion
    if (!hubo_error){
      //temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("arglist");
      prod1 = Get_Primera_Produccion ("expresion");

      prod_aux = Get_Primera_Produccion ("identifier");
      if (prod_aux.Get_Lugar() == "print"){
	prod.Set_Lugar (prod1.Get_Lugar());
	prod.Set_Codigo (prod1.Get_Codigo());
      }
      else {
	
	prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_Codigo()));
	prod.Add_Codigo ("push " + prod1.Get_Lugar());
	
	prod2 = Get_Primera_Produccion ("lista_expresion");
	prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
	
	prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
	prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
	
	// Para saber el nmero de argumentos
	sprintf (Num, "%d", (atoi((prod2.Get_Lugar()).c_str())) +1 );
	prod.Set_Lugar ( Num );
      }
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(95);
  }
  else if (simbolo_gencodigo == G_96_0) { // lista_expresion ::= "," expresion lista_expresion
    if (!hubo_error){
      prod = Get_Segunda_Produccion ("lista_expresion");
      prod2 = Get_Primera_Produccion ("expresion");
      prod.Set_Codigo (prod2.Get_Codigo());
      prod.Add_Codigo ("push " + prod2.Get_Lugar());
      
      prod3 = Get_Primera_Produccion ("lista_expresion");
      // Para saber el nmero de argumentos
      sprintf (Num, "%d", (atoi((prod3.Get_Lugar()).c_str())) +1 );
      prod.Set_Lugar ( Num );

      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Segunda_Produccion (prod);
    }
    Desapilar_Produccion(96);
  }
  else if (simbolo_gencodigo == G_97_0) { // lista_expresion ::= "landa"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("lista_expresion");
      vcodigo.clear();
      prod.Set_Codigo(vcodigo);
      
      // Para saber el nmero de argumentos
      prod.Set_Lugar("0");
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(97);
  }
  else if (simbolo_gencodigo == G_98_0) { // do_statment ::= "do" statment "while" "(" expresion ")" ";"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("do_statment");
      prod1 = Get_Primera_Produccion ("statment");
      prod2 = Get_Primera_Produccion ("expresion");
      etiq = Nueva_EWhile ();
      prod.Add_Codigo ("");
      prod.Add_Codigo (etiq + ":");
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_Codigo()));
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo ("if " + prod2.Get_Lugar() + " == 1 then goto " + etiq);
    
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(98);
  }
  else if (simbolo_gencodigo == G_99_0) { // constructor_declaration ::= "(" opcional_parameter_list ")" statement_block
    if (!hubo_error){
      prod = Get_Primera_Produccion ("constructor_declaration");
      prod1 = Get_Primera_Produccion ("opcional_parameter_list");
      prod2 = Get_Primera_Produccion ("statment_block");

      prod_aux = Get_Primera_Produccion ("lista_field_declaration");
      CLexema cl = tabla_simbolos->Recupera(prod_aux.Get_Clase());
      
      prod.Add_Codigo (cl.GetID() + ":");
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod1.Get_Codigo()));
      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo ("RETURN");
      prod.Add_Codigo ("");
      prod.Add_Codigo ("");
      
      Mostrar_Codigo (prod.Get_Codigo());

      if (opciones->Debug()) {
	Mostrar_Estado_Pila_Entrada();
	Mostrar_Pila_Producciones();
      }

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(99);
  }
  else if (simbolo_gencodigo == G_100_0) { // constante_logica ::= "true"
    if (!hubo_error){    
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("constante_logica");
      prod.Set_Lugar(temp);
      codigo = temp + " = 1";
      prod.Add_Codigo(codigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(100);
  }
  else if (simbolo_gencodigo == G_101_0) { // constante_logica ::= "false"
    if (!hubo_error){
      temp = Nuevo_Temporal();
      prod = Get_Primera_Produccion ("constante_logica");
      prod.Set_Lugar(temp);
      codigo = temp + " = 0";
      prod.Add_Codigo(codigo);
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(101);
  }
  else if (simbolo_gencodigo == G_102_0) { // opcional_identifier ::= "." identifier
    if (!hubo_error){
    }
    Desapilar_Produccion(102);
  }
  else if (simbolo_gencodigo == G_103_0) { // opcional_identifier ::= "landa"
    if (!hubo_error){
    }
    Desapilar_Produccion(103);
  }
  else if (simbolo_gencodigo == G_104_0) { // op_asignacion ::= "="
    if (!hubo_error){
      prod = Get_Primera_Produccion ("op_asignacion");
      prod.Set_Lugar(" = ");
      Set_Primera_Produccion(prod);
    }
    Desapilar_Produccion(104);
  }
  else if (simbolo_gencodigo == G_105_0) { // posible_statment ::= expresion_prima
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_statment");
      prod1 = Get_Primera_Produccion ("expresion_prima");
      
      prod.Set_Codigo (prod1.Get_Codigo());
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_TProd (prod1.Get_TProd());
      
      if (opciones->Debug()){
	cout << "TIPOPRO:" << prod.Get_TProd() << endl;
	Mostrar_Codigo(prod.Get_Codigo());
      }
      
      /*if (prod1.Get_TProd() == "punto"){ Punto ....
	vcodigo.clear();
	if (vcodigo != prod.Get_Codigo()){
	vcodigo = prod.Get_Codigo();
	vcodigo.pop_back();
	vcodigo.pop_back();
	prod.Set_Codigo (vcodigo);
	}
	vobjetos = prod1.Get_LObjetos();
	prod_aux = Get_Primera_Produccion ("identifier");
	CLexema lex_var = tabla_simbolos->Recupera(prod_aux.Get_Lugar());
	CVariableDatos* vd = (CVariableDatos*)lex_var.GetDatos();
	CLexema cl = tabla_simbolos->Recupera(vd->GetTipoDeDatos());
	prod.Add_Codigo ("call " + cl.GetID() + vobjetos[vobjetos.size()-1]);
	temp = Nuevo_Temporal();
	prod.Add_Codigo ("pop " + temp);
	prod.Set_Lugar(temp);
	//prod.Add_Codigo ("llllcall " + prod1.Get_Lugar() + prod2.Get_Lugar());
	//temp = Nuevo_Temporal();
	//prod.Add_Codigo ("pop " + temp);
	//prod.Set_Lugar (temp);
	}*/


      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(105);
  }
  else if (simbolo_gencodigo == G_106_0) { // posible_statment ::= agrupa_expresion
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_statment");
      prod1 = Get_Primera_Produccion ("agrupa_expresion");
      
      prod.Set_Codigo (prod1.Get_Codigo());
      prod.Set_Lugar (prod1.Get_Lugar());
      prod.Set_TProd ("agrupa");
      
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod1.Get_PCodigo()));
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(106);
  }
  else if (simbolo_gencodigo == G_107_0) { // posible_statment ::= op_asignacion expresion
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_statment");
      prod1 = Get_Primera_Produccion ("op_asignacion");
      prod2 = Get_Primera_Produccion ("expresion");
      
      prod.Set_Codigo (prod2.Get_Codigo());
      prod.Set_Lugar (prod1.Get_Lugar() + prod2.Get_Lugar());
      prod.Set_TProd ("asig");
    
      prod.Set_PCodigo (Concatenar_Vectores(prod.Get_PCodigo(), prod2.Get_PCodigo()));
      
      Mostrar_Codigo (prod.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(107);
  }
  else if (simbolo_gencodigo == G_108_0) { // posible_statment ::= identifier variable_declaration
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_statment");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("variable_declaration");

      prod.Set_Codigo (Concatenar_Vectores (prod.Get_Codigo(), prod2.Get_Codigo()));
      
      
      prod.Set_TProd ("obj");

      
      prod.Set_LObjetos (prod2.Get_LObjetos());
      if (prod2.Get_Lugar() == "")
	prod.Add_Objeto (prod1.Get_Lugar());

      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(108);
  }
  else if (simbolo_gencodigo == G_109_0) { // statment ::= "const" posible_const;
    if (!hubo_error){
      prod = Get_Primera_Produccion ("statment");
      prod1 = Get_Primera_Produccion ("posible_const");

      prod.Set_Codigo(prod1.Get_Codigo());
      
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(109);
  }
  else if (simbolo_gencodigo == G_110_0) { // posible_const ::= "int" identifier "=" integer_literal
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_const");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("integer_literal");
      
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo(prod1.Get_Lugar() + " = " + prod2.Get_Lugar());
          
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(110);
  }
  else if (simbolo_gencodigo == G_111_0) { // posible_const ::= "boolean" identifier "=" constante_logica
    if (!hubo_error){
      prod = Get_Primera_Produccion ("posible_const");
      prod1 = Get_Primera_Produccion ("identifier");
      prod2 = Get_Primera_Produccion ("constante_logica");
      
      prod.Set_Codigo (Concatenar_Vectores(prod.Get_Codigo(), prod2.Get_Codigo()));
      prod.Add_Codigo(prod1.Get_Lugar() + " = " + prod2.Get_Lugar());
            
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(111);
  }
  else if (simbolo_gencodigo == G_112_0) { // variable_declarator ::= "(" opcional_arglist ")"
    if (!hubo_error){
      prod = Get_Primera_Produccion ("variable_declarator");
      prod1 = Get_Primera_Produccion ("opcional_arglist");
      
      prod.Set_Codigo (prod1.Get_Codigo());

      prod.Set_TProd("obj");

      Mostrar_Codigo (prod.Get_Codigo());

      prod.Set_Lugar("()");
      Set_Primera_Produccion (prod);
    }
    Desapilar_Produccion(112);
  }
}

//
// GET_PRIMERA_PRODUCCION
//
CProduccion 
CCompilador::Get_Primera_Produccion(const string& a) const
{
  stack<CProduccion> pp = pila_producciones;

  while (!pp.empty()) {
    if (pp.top().Antecedente() == a)
      return pp.top();
    else
      pp.pop();
  }

  return CProduccion();
}

//
// SET_PRIMERA_PRODUCCION
//
void
CCompilador::Set_Primera_Produccion(const CProduccion& p)
{
  stack<CProduccion> p_aux;

  while(pila_producciones.top().Antecedente() != p.Antecedente()) {
    p_aux.push(pila_producciones.top());
    pila_producciones.pop();
  }

  // Cambio.
  pila_producciones.pop();
  pila_producciones.push(p);

  while(!p_aux.empty()) {
    pila_producciones.push(p_aux.top());
    p_aux.pop();
  }
}

//
// GET_SEGUNDA_PRODUCCION
//
CProduccion 
CCompilador::Get_Segunda_Produccion(const string& a) const
{
  int primer = 0;
  stack<CProduccion> pp = pila_producciones;
  
  while (!pp.empty()) {
    if (pp.top().Antecedente() == a)
      if (primer)
	return pp.top();
      else
	primer = 1;
    else
      pp.pop();
  }

  return CProduccion();
}

//
// SET_SEGUNDA_PRODUCCION
//
void
CCompilador::Set_Segunda_Produccion(const CProduccion& p)
{
  stack<CProduccion> p_aux;

  while(pila_producciones.top().Antecedente() != p.Antecedente()) {
    p_aux.push(pila_producciones.top());
    pila_producciones.pop();
  }
  //p_aux.push(pila_producciones.top());
  pila_producciones.pop(); 
  while(pila_producciones.top().Antecedente() != p.Antecedente()) {
    p_aux.push(pila_producciones.top());
    pila_producciones.pop();
  }
  // Cambio. 
  pila_producciones.pop();
  pila_producciones.push(p);

  while(!p_aux.empty()) {
    pila_producciones.push(p_aux.top());
    p_aux.pop();
  }
}

//
// DESAPILAR_PRODUCCION
//
void
CCompilador::Desapilar_Produccion(int prod)
{
  while (pila_producciones.top().Antecedente() != 
	 gramatica[prod - 1].Antecedente())
    pila_producciones.pop();
}


//
// FUNCIONES PARA LA GENERACIN DE CDIGO INTERMEDIO
//
string
CCompilador::Nuevo_Temporal(void)
{
  string tem = "tem";
  char Num[3];

  sprintf (Num, "%d", ++NTemporal);
  
  tem += Num;
  if (opciones->Debug())
    cout << tem << endl;
  return tem;  
}

string
CCompilador::Nueva_EWhile(void)
{
  string etiq = "WHILE";
  char Num[3];

  sprintf (Num, "%d", ++NEWhile);

  etiq += Num;
  if (opciones->Debug())
    cout << etiq << endl;
  return etiq;
}

void
CCompilador::Escribir_Codigo (vector<string> vcod)
{
  const char* linea = new char[70];
  string codigo;
  vector<string> vtemp;
  
  while (!vcod.empty()){
    codigo = vcod[vcod.size()-1];
    vtemp.push_back(codigo);
    vcod.pop_back();
  }

  if (opciones->Debug())
    cout << "GGGGGGGGGG" << endl;

  salida = fopen(opciones->Fichero_Codigo_Intermedio(), "w");
  if (!salida)
    throw Argumentos_Incorrectos_Ex("Error al abrir el fichero para codigo intermedio.");
  while (!vtemp.empty()){
    codigo = vtemp[vtemp.size()-1];
    if (opciones->Debug())
      cout << codigo << endl;
    linea = codigo.c_str();
    fprintf (salida, "%s\n", linea);
    vtemp.pop_back();
  }
  fclose (salida);
  
}

vector<string>
CCompilador::Concatenar_Vectores (vector<string> vdestino, vector<string> vorigen)
{
  string codigo;
  vector<string> vtemp;
  
  vtemp.clear();
  while (!vorigen.empty()){
    codigo = vorigen[vorigen.size()-1];
    vtemp.push_back (codigo);
    vorigen.pop_back();
  }

  while (!vtemp.empty()){
    codigo = vtemp[vtemp.size()-1];
    vdestino.push_back (codigo);
    vtemp.pop_back();
  }
  return (vdestino);
}

void 
CCompilador::Mostrar_Codigo (vector<string> vcodigo)
{
  string codigo;  
  if (opciones->Debug()){
    cout << "CCCCCCCOOOD:" << endl;
    while (!vcodigo.empty()){
      codigo = vcodigo[vcodigo.size()-1];
      cout << codigo << endl;
      vcodigo.pop_back();
    }
  }
}
