// $Id: CCompilador.hpp,v 1.9 2001/09/24 19:29:30 chapi Exp $

#if !defined( _CCompilador_HPP_ )
#define _CCompilador_HPP_

#include <FlexLexer.h>
#include <stack>
#include <fstream>
#include <algorithm>
#include "data.hpp"
#include "CTablaSimbolos.hpp"
#include "COpciones.hpp"
#include "CProduccion.hpp"
#include "stdio.h"

class CCompilador {
public:
  CCompilador(const COpciones& op);
  ~CCompilador(void);
  void Compilar(void);
private:
  FILE*              salida;              // Fichero de salida para el cdigo intermedio.
  int                NTemporal;           // Nmero de variable temporal. Para la Gen. Cod.
  int                NEWhile;             // Nmero de etiqueta para los bucles while.

  COpciones*         opciones;            // Opciones del compilador.
  ifstream*          entrada;             // Fichero a compilar.
  NoTerminales       no_terminales;       // Conjunto de no terminales.
  Terminales         terminales;          // Conjunto de terminales.
  SimbolosSemanticos simbolos_semanticos; // Simbolos para el anlisis semntico.
  SimbolosGenCodigo  simbolos_gencodigo;
  TablaAS            tabla_as;            // Tabla LL(1).
  Gramatica          gramatica;           // Gramtica utilizada.
  CTablaSimbolos*    tabla_simbolos;      // Tabla de smbolos.
  stack<CProduccion> pila_producciones;   // Pila de producciones.
  stack<string>      pila_gramatical;     // Pila de smbolos gramaticales.
  yyFlexLexer*       analizador_lexico;   // Analizador lxico.
  int                token;               // Entrada devuelta por el analizador lxico.
  bool               hay_main;            // Flag para el procedimiento principal.
  bool               hubo_error;          // Notificacin de error.

  void Derivar(int nprod);
  void Mostrar_Estado_Pila_Entrada(void) const;
  void Mostrar_Pila_Producciones(void) const;
  void Analisis_Semantico(const string& simbolo_semantico);
  void Generacion_Codigo(const string& simbolo_gencodigo);
  CProduccion Get_Primera_Produccion(const string& a) const;
  void Set_Primera_Produccion(const CProduccion& p);
  CProduccion Get_Segunda_Produccion(const string& a) const;
  void Set_Segunda_Produccion(const CProduccion& p);
    void Desapilar_Produccion(int nprod);
  string Nuevo_Temporal(void);
  string Nueva_EWhile(void);
  void Escribir_Codigo (vector<string> vcod);
  vector<string> Concatenar_Vectores (vector<string> vdestino, vector<string> vorigen);
  void Mostrar_Codigo (vector<string> vcodigo);
};

#endif // _CCompilador_HPP_
