// $Id: CLexema.hpp,v 1.15 2001/09/25 07:32:41 jpl Exp $

#if !defined( _CLexema_HPP_ )
#define _CLexema_HPP_

#include <string>
#include <list>
#include <algorithm>
#include "data.hpp"

//
// CVACIO
//
class CVacio {
public:
  CVacio(void) {}
};

class CVariableDatos;
class CConstanteDatos;
class CFuncionDatos;
class CClaseDatos;

//
// CLEXEMA
//
class CLexema {
private:
  string identificador;
  TipoLexema tipo;
  union {
    CVacio*          v;   // tipo == L_VACIO
    CVariableDatos*  vd;  // tipo == L_VARIABLE
    CConstanteDatos* cod; // tipo == L_CONSTANTE
    CFuncionDatos*   fd;  // tipo == L_FUNCION
    CClaseDatos*     cld; // tipo == L_CLASE
  };
public:
  CLexema(void) 
  {
    identificador = "";
    tipo = L_VACIO;
    v = new CVacio();
  }
  CLexema(string s, TipoLexema t, void* d)
  { 
    identificador = s;
    tipo = t;
    switch (t) {
    case L_VARIABLE:
      vd = (CVariableDatos*)d;
    case L_CONSTANTE:
      cod = (CConstanteDatos*)d;
    case L_FUNCION:
      fd = (CFuncionDatos*)d;
    case L_CLASE:
      cld = (CClaseDatos*)d;
    case L_VACIO:
      v  = (CVacio*)d;
    }
  }
  inline string GetID(void) const { return identificador; }
  inline TipoLexema GetTipo(void) const { return tipo; }
  inline void* GetDatos(void) const 
  { 
    switch (tipo) {
    case L_VARIABLE:
      return (void*)vd;
    case L_CONSTANTE:
      return (void*)vd;
    case L_FUNCION:
      return (void*)fd;
    case L_CLASE:
      return (void*)cld;
    case L_VACIO:
    default:
      return (void*)v;
    }
  }
  inline void SetLexema(const string& s, TipoLexema t, void* d)
  {
    identificador = s;
    tipo = t;
    switch (t) {
    case L_VARIABLE:
      vd = (CVariableDatos*)d;
    case L_CONSTANTE:
      cod = (CConstanteDatos*)d;
    case L_FUNCION:
      fd = (CFuncionDatos*)d;
    case L_CLASE:
      cld = (CClaseDatos*)d;
    case L_VACIO:
      v  = (CVacio*)v;
    }
  }
};

//
// CVARIABLEDATOS
//
class CVariableDatos {
private:
  string tipo_de_datos;
  string valor;
public:
  CVariableDatos(void) { tipo_de_datos = ""; valor = ""; }
  CVariableDatos(string tdd, string v)  { tipo_de_datos = tdd; valor = v; }
  inline string GetTipoDeDatos(void) const { return tipo_de_datos; }
  void SetValor(const string& v) { valor = v; }
  inline string GetValor(void) const { return valor; } 
};

//
// CCONSTANTEDATOS
//
class CConstanteDatos {
private:
  string tipo_de_datos;
  string valor;
public:
  CConstanteDatos(void) { tipo_de_datos = ""; valor = ""; }
  CConstanteDatos(string tdd, string v)  { tipo_de_datos = tdd; valor = v; }
  inline string GetTipoDeDatos(void) const { return tipo_de_datos; }
  void SetValor(const string& v) { valor = v; }
  inline string GetValor(void) const { return valor; }
};

//
// CFUNCIONDATOS
//
class CFuncionDatos {
private:
  string tipo_de_datos;
  list<string> parametros;
public:
  CFuncionDatos(void) {}
  CFuncionDatos(const string& td, const list<string>& p)
  {
    tipo_de_datos = td;
    parametros = p;
  }
  inline string GetTipoDeDatos(void) const { return tipo_de_datos; }
  inline list<string> GetParametros(void) const { return parametros; }
};

//
// CCLASEDATOS
//
class CClaseDatos {
private:
  TipoAcceso          acceso;
  list<Tipo_Atributo> atributos;
  list<Tipo_Metodo>   metodos;
public:
  CClaseDatos(TipoAcceso acc) { acceso = acc; }
  CClaseDatos(TipoAcceso acc, const list<Tipo_Atributo>& atr, 
	      const list<Tipo_Metodo>& met)
  {
    acceso = acc;
    atributos = atr;
    metodos = met;
  }
  inline TipoAcceso Get_Acceso(void) const { return acceso; }
  inline list<Tipo_Atributo> Get_Atributos(void) const { return atributos; }
  void Inserta_Metodo(const Tipo_Metodo& m) { metodos.push_back(m); }
  bool Es_Metodo(const string& id)
  {
    list<Tipo_Metodo>::iterator it;
    it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(id));
    return (it != metodos.end());
  }
  bool Es_Metodo_Privado(const string& id)
  {
    list<Tipo_Metodo>::iterator it;
    it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(id));
    return (it->acceso == A_PRIVATE);
  }
  string Get_Tipo_Metodo(const string& id)
  {
    list<Tipo_Metodo>::iterator it;
    it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(id));
    return it->tipo;
  }
  void Inserta_Atributo(const Tipo_Atributo& a) { atributos.push_back(a); }
  bool Es_Atributo(const string& id)
  {
    list<Tipo_Atributo>::iterator it;
    it = find_if(atributos.begin(), atributos.end(), ID_Atributos_Iguales(id));
    return (it != atributos.end());
  }  
  bool Es_Atributo_Privado(const string& id)
  {
    list<Tipo_Atributo>::iterator it;
    it = find_if(atributos.begin(), atributos.end(), ID_Atributos_Iguales(id));
    return (it->acceso == A_PRIVATE);
  }
  string Get_Tipo_Atributo(const string& id)
  {
    list<Tipo_Atributo>::iterator it;
    it = find_if(atributos.begin(), atributos.end(), ID_Atributos_Iguales(id));
    return it->tipo;
  }
  void Inserta_Parametro(const string& m_id, const Tipo_Parametro& p)
  { 
    list<Tipo_Metodo>::iterator it;
    it = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(m_id));
    if (it != metodos.end())
      it->parametros.push_back(p);
  }
  Tipo_Parametro Get_NParametro(const string& m_id, int n_parametro)
  {
    list<Tipo_Metodo>::iterator itm;
    itm = find_if(metodos.begin(), metodos.end(), ID_Metodos_Iguales(m_id));

    list<Tipo_Parametro>::iterator itp = itm->parametros.begin();
    for (int i = 0; i < (n_parametro - 1); i++) itp++;
    return *itp;
  }
  inline list<Tipo_Metodo> Get_Metodos(void) const { return metodos; }
};

#endif // _CLexema_HPP_
