// $Id: CTablaSimbolos.cpp,v 1.4 2001/09/23 17:58:03 jpl Exp $

#include "CTablaSimbolos.hpp"

//
// CTABLASIMBOLOS
//
CTablaSimbolos::CTablaSimbolos(void)
{
  // Introducimos las palabras reservadas en la tabla.
  palabras_reservadas.insert(reservadas, 
			     reservadas + NUM_PALABRAS_RESERVADAS);

  // Inicializamos el mbito.
  bloques.push(0);
}

//
// INSERTA
//
void 
CTablaSimbolos::Inserta(const CLexema& lexema)
{
  // Comprobamos que el lexema, no es un palabra reservada.
  if (palabras_reservadas.find(lexema.GetID()) == palabras_reservadas.end()) {
    // Introducimos en la tabla de smbolos.
    ambito.push(lexema);
    
    // Aumentamos el alcance del mbito.
    int nuevo_alcance = bloques.top();
    bloques.pop();
    bloques.push(++nuevo_alcance);
  }
  else 
    throw Palabra_Reservada_Ex();
}

//
// BUSCA
//
bool 
CTablaSimbolos::Busca(const string& id_lexema) const
{
  // Si es palabra reservada, entonces est.
  if (palabras_reservadas.find(id_lexema) == palabras_reservadas.end()) {
    // Utilizamos una copia del mbito para la bsqueda.
    stack<CLexema> ambito_copia = ambito;
    CLexema lexema_actual;
    bool encontrado = false;
    
    while (ambito_copia.size() && !encontrado) {
      lexema_actual = ambito_copia.top();
      encontrado = lexema_actual.GetID() == id_lexema;
      ambito_copia.pop();
    }
    
    return encontrado;
  }
  else 
    return true;
}

//
// RECUPERA
//
CLexema 
CTablaSimbolos::Recupera(const string& id_lexema) const
{
  // Utilizamos una copia del mbito para la modificacin.
  stack<CLexema> ambito_copia = ambito;
  CLexema lexema_actual;
  bool encontrado = false;
  
  while (ambito_copia.size() && !encontrado) {
    lexema_actual = ambito_copia.top();
    encontrado = lexema_actual.GetID() == id_lexema;
    ambito_copia.pop();
  }
  
  // Si no el lexema, se devuelve uno vaco.
  if (encontrado) 
    return lexema_actual;
  else
    return CLexema();
}

//
// MODIFICA
//
void 
CTablaSimbolos::Modifica(const CLexema& lexema)
{
  // Comprobamos que el lexema, no es un palabra reservada.
  if (palabras_reservadas.find(lexema.GetID()) == palabras_reservadas.end()) {      
    // Utilizamos una copia del mbito para la modificacin.
    stack<CLexema> ambito_copia;
    CLexema lexema_actual;
    bool encontrado = false;
    
    // Vamos desapilando hasta encontrar el lexema si existe.
    while (ambito.size() && !encontrado) {
      lexema_actual = ambito.top();
      encontrado = lexema_actual.GetID() == lexema.GetID();
      if (!encontrado) {
	ambito_copia.push(ambito.top());
	ambito.pop();
      }
      else {
	ambito.pop();
	ambito.push(lexema);
      }	  
    }
    
    // Volvemos a apilar. 
    while (ambito_copia.size())	{
      ambito.push(ambito_copia.top());
      ambito_copia.pop();
    }
  }
  else 
    throw Palabra_Reservada_Ex();
}

//
// PUSHBLOQUE
//
void 
CTablaSimbolos::PushBloque(void)
{
  // Introducir el siguiente nivel de mbito.
  bloques.push(bloques.top()++);
}

//
// POPBLOQUE
//
void 
CTablaSimbolos::PopBloque(void)
{
  // Obtenemos el lmite actual del mbito y desapilamos.
  int ambito_superior = bloques.top();
  bloques.pop();

  // Nos olvidamos del anterior mbito.
  for (int i = 0; i < (ambito_superior - bloques.top()); i++)
    ambito.pop();
}
