// $Id: data.hpp,v 1.30 2001/09/25 18:53:34 jpl Exp $

#if !defined( _Data_HPP_ )
#define _Data_HPP_

#include <string>
#include <set>
#include <map>
#include <vector>
#include <list>

typedef set<string>                 Terminales;
typedef set<string>                 NoTerminales;
typedef set<string>                 SimbolosSemanticos;
typedef set<string>                 SimbolosGenCodigo;
typedef map<string, map<int, int> > TablaAS;

typedef enum { 
  L_VACIO, 
  L_VARIABLE,
  L_CONSTANTE,
  L_FUNCION, 
  L_CLASE 
} TipoLexema;

typedef enum { 
  A_PRIVATE, 
  A_PUBLIC 
} TipoAcceso;

typedef enum { 
  ID_CONSTANTE, 
  ID_VARIABLE 
} TipoIdentificador;

typedef struct {
  string id;
  string tipo;
} Tipo_Parametro;

typedef struct {
  TipoAcceso           acceso;
  string               tipo;
  string               id;
  list<Tipo_Parametro> parametros;
} Tipo_Metodo;

class ID_Metodos_Iguales 
  : public unary_function<Tipo_Metodo, bool> {
  string s;
public:
  explicit ID_Metodos_Iguales(const string& ss) : s(ss) {}
  bool operator() (const Tipo_Metodo& m) const { return m.id == s; }
};

typedef struct {
  TipoAcceso acceso;
  string     tipo;
  string     id;
} Tipo_Atributo;  

class ID_Atributos_Iguales 
  : public unary_function<Tipo_Atributo, bool> {
  string s;
public:
  explicit ID_Atributos_Iguales(const string& ss) : s(ss) {}
  bool operator() (const Tipo_Atributo& a) const { return a.id == s; }
};

// Tokens reconocidos por el analizador lxico.
const int NO_RECONOCIDO       = 0;
const int COMENTARIO_NLINEAS  = 1;
const int COMENTARIO_1LINEA   = 2;
const int OCTAL               = 3;
const int DECIMAL             = 4;
const int HEXADECIMAL         = 5;
const int TRUE                = 6;
const int FALSE               = 7;
const int CADENA_CARACTERES   = 8;
const int OP_INCREMENTO       = 9;
const int OP_DECREMENTO       = 10;
const int OP_MAS              = 11;
const int OP_MENOS            = 12;
const int OP_POR              = 13;
const int OP_DIVIDIR          = 14;
const int OP_MODULO           = 15;
const int OP_IGUAL_IGUAL      = 16;
const int OP_DISTINTO         = 17;
const int OP_MENOR            = 18;
const int OP_MAYOR            = 19;
const int OP_MENOR_IGUAL      = 20;
const int OP_MAYOR_IGUAL      = 21;
const int OP_AND              = 22;
const int OP_OR               = 23;
const int OP_ASIGNACION       = 24;
const int OP_MIEMBRO          = 25;
const int TD_ENTERO           = 26;
const int TD_BOOLEAN          = 27;
const int TD_VOID             = 28;
const int INSTR_ENTRADA       = 29;
const int INSTR_SALIDA        = 30;
const int PUNTO_Y_COMA        = 31;
const int COMA                = 32;
const int PARENTESIS_ABRIR    = 33;
const int PARENTESIS_CERRAR   = 34;
const int LLAVES_ABRIR        = 35;
const int LLAVES_CERRAR       = 36;
const int CORCHETES_ABRIR     = 37;
const int CORCHETES_CERRAR    = 38;
const int RETURN              = 39;
const int CLASS               = 40;
const int NEW                 = 41;
const int PUBLIC              = 42;
const int PRIVATE             = 43;
const int WHILE               = 44;
const int DO                  = 45;
const int MAIN                = 46;
const int IDENTIFICADOR       = 47;
const int BLANCO              = 48;
const int CONST               = 49;
const int $                   = 50;

// No terminales. Cogidos de gramatica_bnf_modificada.txt.
const string NT_COMPILATION_UNIT              = "compilation_unit";
const string NT_LISTA_TYPE_DECLARATION        = "lista_type_declaration";
const string NT_TYPE_DECLARATION              = "type_declaration";
const string NT_CLASS_DECLARATION             = "class_declaration";
const string NT_LISTA_FIELD_DECLARATION       = "lista_field_declaration";
const string NT_MODIFIER                      = "modifier";
const string NT_IDENTIFIER                    = "identifier";
const string NT_FIELD_DECLARATION             = "field_declaration";
const string NT_FIELD_DECLARATION_PRIMA       = "field_declaration_prima";
const string NT_FIELD_DECLARATION_PRIMA_PRIMA = "field_declaration_prima_prima";
const string NT_METHOD_DECLARATION            = "method_declaration";
const string NT_OPCIONAL_PARAMETER_LIST       = "opcional_parameter_list";
const string NT_LISTA_CORCHETES               = "lista_corchetes";
const string NT_METHOD_DECLARATION_FINAL      = "method_declaration_final";
const string NT_TYPE                          = "type";
const string NT_TYPE_SPECIFIER                = "type_specifier";
const string NT_CLASS_NAME                    = "class_name";
const string NT_PARAMETER_LIST                = "parameter_list";
const string NT_LISTA_PARAMETER_LIST          = "lista_parameter_list";
const string NT_PARAMETER                     = "parameter";
const string NT_STATMENT_BLOCK                = "statment_block";
const string NT_LISTA_STATMENT                = "lista_statment";
const string NT_STATMENT                      = "statment";
const string NT_OPCIONAL_EXPRESION            = "opcional_expresion";
const string NT_VARIABLE_DECLARATION          = "variable_declaration";
const string NT_LISTA_VARIABLE_DECLARATOR     = "lista_variable_declarator";
const string NT_VARIABLE_DECLARATOR           = "variable_declarator";
const string NT_OPCIONAL_VARIABLE_INITIALIZER = "opcional_variable_initializer";
const string NT_VARIABLE_INITIALIZER          = "variable_initializer";
const string NT_EXPRESION                     = "expresion";
const string NT_EXPRESION_PRIMA               = "expresion_prima";
const string NT_EXPRESION_1                   = "expresion_1";
const string NT_EXPRESION_1_PRIMA             = "expresion_1_prima";
const string NT_EXPRESION_2                   = "expresion_2";
const string NT_EXPRESION_2_PRIMA             = "expresion_2_prima";
const string NT_EXPRESION_3                   = "expresion_3";
const string NT_EXPRESION_3_PRIMA             = "expresion_3_prima";
const string NT_EXPRESION_4                   = "expresion_4";
const string NT_EXPRESION_5                   = "expresion_5";
const string NT_OP_INDECREMENTO               = "op_indecremento";
const string NT_OP_ARITMETICO                 = "op_aritmetico";
const string NT_OP_RELACIONAL                 = "op_relacional";
const string NT_OP_LOGICO                     = "op_logico";
const string NT_AGRUPA_EXPRESION              = "agrupa_expresion";
const string NT_OPCIONAL_ARGLIST              = "opcional_arglist";
const string NT_LITERAL_EXPRESION             = "literal_expresion";
const string NT_INTEGER_LITERAL               = "integer_literal";
const string NT_STRING                        = "string";
const string NT_ARGLIST                       = "arglist";
const string NT_LISTA_EXPRESION               = "lista_expresion";
const string NT_DO_STATMENT                   = "do_statment";
const string NT_CONSTRUCTOR_DECLARATION       = "constructor_declaration";
const string NT_CONSTANTE_LOGICA              = "constante_logica";
const string NT_OPCIONAL_IDENTIFIER           = "opcional_identifier";
const string NT_OP_ASIGNACION                 = "op_asignacion";
const string NT_POSIBLE_STATMENT              = "posible_statment";
const string NT_POSIBLE_CONST                 = "posible_const";

// Terminales.
const string T_LLAVES_ABRIR       = "{";
const string T_LLAVES_CERRAR      = "}";
const string T_PARENTESIS_ABRIR   = "(";
const string T_PARENTESIS_CERRAR  = ")";
const string T_CORCHETES_ABRIR    = "[";
const string T_CORCHETES_CERRAR   = "]";

const string T_PUBLIC             = "public";
const string T_PRIVATE            = "private";
const string T_CLASS              = "class";
const string T_RETURN             = "return";

const string T_PUNTO_Y_COMA       = ";";
const string T_OP_MIEMBRO         = ".";
const string T_COMA               = ",";
const string T_OP_ASIGNACION      = "=";

const string T_TD_VOID            = "void";
const string T_TD_BOOLEAN         = "boolean";
const string T_TD_INT             = "int";
const string T_TD_CADENA          = "cadena";

const string T_OP_INCREMENTO      = "++";
const string T_OP_DECREMENTO      = "--";

const string T_TRUE               = "true";
const string T_FALSE              = "false";

const string T_OP_MAS             = "+";
const string T_OP_MENOS           = "-";
const string T_OP_POR             = "*";
const string T_OP_DIVIDIR         = "/";
const string T_OP_MODULO          = "%";

const string T_OP_MAYOR           = ">";
const string T_OP_MENOR           = "<";
const string T_OP_MAYOR_IGUAL     = ">=";
const string T_OP_MENOR_IGUAL     = "<=";
const string T_OP_IGUAL_IGUAL     = "==";
const string T_OP_DISTINTO        = "!=";

const string T_OP_AND             = "&&";
const string T_OP_OR              = "||";

const string T_DO                 = "do";
const string T_WHILE              = "while";

const string T_OCTAL              = "octal";
const string T_DECIMAL            = "decimal";
const string T_HEXADECIMAL        = "hexadecimal";
const string T_CADENA_CARACTERES  = "cadena_caracteres";

const string T_CONST              = "const";

// Smbolos para el semntico.
const string S_1_0 = "s_1_0";
const string S_1_1 = "s_1_1";
const string S_2_0 = "s_2_0";
const string S_3_0 = "s_3_0";
const string S_4_0 = "s_4_0";
const string S_5_0 = "s_5_0";
const string S_5_1 = "s_5_1";
const string S_5_2 = "s_5_2";
const string S_6_0 = "s_6_0";
const string S_6_1 = "s_6_1";
const string S_6_2 = "s_6_2";
const string S_7_0 = "s_7_0";
const string S_8_0 = "s_8_0";
const string S_9_0 = "s_9_0";
const string S_10_0 = "s_10_0";
const string S_11_0 = "s_11_0";
const string S_12_0 = "s_12_0";
const string S_12_1 = "s_12_1";
const string S_13_0 = "s_13_0";
const string S_14_0 = "s_14_0";
const string S_14_1 = "s_14_1";
const string S_15_0 = "s_15_0";
const string S_15_1 = "s_15_1";
const string S_16_0 = "s_16_0";
const string S_16_1 = "s_16_1";
const string S_17_0 = "s_17_0";
const string S_17_1 = "s_17_1";
const string S_18_0 = "s_18_0";
const string S_18_1 = "s_18_1";
const string S_18_2 = "s_18_2";
const string S_19_0 = "s_19_0";
const string S_19_1 = "s_19_1";
const string S_20_0 = "s_20_0";
const string S_21_0 = "s_21_0";
const string S_22_0 = "s_22_0";
const string S_23_0 = "s_23_0";
const string S_23_1 = "s_23_1";
const string S_24_0 = "s_24_0";
const string S_25_0 = "s_25_0";
const string S_26_0 = "s_26_0";
const string S_27_0 = "s_27_0";
const string S_28_0 = "s_28_0";
const string S_29_0 = "s_29_0";
const string S_30_0 = "s_30_0";
const string S_31_0 = "s_31_0";
const string S_31_1 = "s_31_1";
const string S_32_0 = "s_32_0";
const string S_32_1 = "s_32_1";
const string S_33_0 = "s_33_0";
const string S_34_0 = "s_34_0";
const string S_34_1 = "s_34_1";
const string S_35_0 = "s_35_0";
const string S_35_1 = "s_35_1";
const string S_36_0 = "s_36_0";
const string S_36_1 = "s_36_1";
const string S_36_2 = "s_36_2";
const string S_37_0 = "s_37_0";
const string S_38_0 = "s_38_0";
const string S_38_1 = "s_38_1";
const string S_39_0 = "s_39_0";
const string S_39_1 = "s_39_1";
const string S_40_0 = "s_40_0";
const string S_40_1 = "s_40_1";
const string S_41_0 = "s_41_0";
const string S_41_1 = "s_41_1";
const string S_42_0 = "s_42_0";
const string S_42_1 = "s_42_1";
const string S_43_0 = "s_43_0";
const string S_43_1 = "s_43_1";
const string S_44_0 = "s_44_0";
const string S_44_1 = "s_44_1";
const string S_45_0 = "s_45_0";
const string S_46_0 = "s_46_0";
const string S_46_1 = "s_46_1";
const string S_47_0 = "s_47_0";
const string S_48_0 = "s_48_0";
const string S_48_1 = "s_48_1";
const string S_48_2 = "s_48_2";
const string S_49_0 = "s_49_0";
const string S_49_1 = "s_49_1";
const string S_49_2 = "s_49_2";
const string S_50_0 = "s_50_0";
const string S_51_0 = "s_51_0";
const string S_51_1 = "s_51_1";
const string S_52_0 = "s_52_0";
const string S_53_0 = "s_53_0";
const string S_54_0 = "s_54_0";
const string S_55_0 = "s_55_0";
const string S_55_1 = "s_55_1";
const string S_55_2 = "s_55_2";
const string S_56_0 = "s_56_0";
const string S_56_1 = "s_56_1";
const string S_57_0 = "s_57_0";
const string S_57_1 = "s_57_1";
const string S_57_2 = "s_57_2";
const string S_58_0 = "s_58_0";
const string S_58_1 = "s_58_1";
const string S_58_2 = "s_58_2";
const string S_59_0 = "s_59_0";
const string S_59_1 = "s_59_1";
const string S_59_2 = "s_59_2";
const string S_60_0 = "s_60_0";
const string S_60_1 = "s_60_1";
const string S_60_2 = "s_60_2";
const string S_61_0 = "s_61_0";
const string S_62_0 = "s_62_0";
const string S_62_1 = "s_62_1";
const string S_62_2 = "s_62_2";
const string S_63_0 = "s_63_0";
const string S_63_1 = "s_63_1";
const string S_63_2 = "s_63_2";
const string S_64_0 = "s_64_0";
const string S_65_0 = "s_65_0";
const string S_66_0 = "s_66_0";
const string S_67_0 = "s_67_0";
const string S_68_0 = "s_68_0";
const string S_69_0 = "s_69_0";
const string S_69_1 = "s_69_1";
const string S_70_0 = "s_70_0";
const string S_71_0 = "s_71_0";
const string S_72_0 = "s_72_0";
const string S_73_0 = "s_73_0";
const string S_74_0 = "s_74_0";
const string S_75_0 = "s_75_0";
const string S_76_0 = "s_76_0";
const string S_77_0 = "s_77_0";
const string S_78_0 = "s_78_0";
const string S_79_0 = "s_79_0";
const string S_80_0 = "s_80_0";
const string S_81_0 = "s_81_0";
const string S_82_0 = "s_82_0";
const string S_83_0 = "s_83_0";
const string S_84_0 = "s_84_0";
const string S_85_0 = "s_85_0";
const string S_85_1 = "s_85_1";
const string S_86_0 = "s_86_0";
const string S_87_0 = "s_87_0";
const string S_87_1 = "s_87_1";
const string S_88_0 = "s_88_0";
const string S_89_0 = "s_89_0";
const string S_90_0 = "s_90_0";
const string S_91_0 = "s_91_0";
const string S_92_0 = "s_92_0";
const string S_93_0 = "s_93_0";
const string S_94_0 = "s_94_0";
const string S_95_0 = "s_95_0";
const string S_95_1 = "s_95_1";
const string S_95_2 = "s_95_2";
const string S_96_0 = "s_96_0";
const string S_96_1 = "s_96_1";
const string S_97_0 = "s_97_0";
const string S_98_0 = "s_98_0";
const string S_98_1 = "s_98_1";
const string S_98_2 = "s_98_2";
const string S_98_3 = "s_98_3";
const string S_99_0 = "s_99_0";
const string S_99_1 = "s_99_1";
const string S_99_2 = "s_99_2";
const string S_100_0 = "s_100_0";
const string S_101_0 = "s_101_0";
const string S_102_0 = "s_102_0";
const string S_102_1 = "s_102_1";
const string S_102_2 = "s_102_2";
const string S_103_0 = "s_103_0";
const string S_104_0 = "s_104_0";
const string S_105_0 = "s_105_0";
const string S_105_1 = "s_105_1";
const string S_106_0 = "s_106_0";
const string S_106_1 = "s_106_1";
const string S_107_0 = "s_107_0";
const string S_107_1 = "s_107_1";
const string S_107_2 = "s_107_2";
const string S_108_0 = "s_108_0";
const string S_108_1 = "s_108_1";
const string S_109_0 = "s_109_0";
const string S_110_0 = "s_110_0";
const string S_111_0 = "s_111_0";
const string S_112_0 = "s_112_0";

// Smbolos para la generacin de cdigo
const string G_1_0 = "g_1_0";
const string G_2_0 = "g_2_0";
const string G_3_0 = "g_3_0";
const string G_4_0 = "g_4_0";
const string G_5_0 = "g_5_0";
const string G_6_0 = "g_6_0";
const string G_7_0 = "g_7_0";
const string G_8_0 = "g_8_0";
const string G_9_0 = "g_9_0";
const string G_10_0 = "g_10_0";
const string G_11_0 = "g_11_0";
const string G_12_0 = "g_12_0";
const string G_13_0 = "g_13_0";
const string G_14_0 = "g_14_0";
const string G_15_0 = "g_15_0";
const string G_16_0 = "g_16_0";
const string G_17_0 = "g_17_0";
const string G_18_0 = "g_18_0";
const string G_19_0 = "g_19_0";
const string G_20_0 = "g_20_0";
const string G_21_0 = "g_21_0";
const string G_22_0 = "g_22_0";
const string G_23_0 = "g_23_0";
const string G_24_0 = "g_24_0";
const string G_25_0 = "g_25_0";
const string G_26_0 = "g_26_0";
const string G_27_0 = "g_27_0";
const string G_28_0 = "g_28_0";
const string G_29_0 = "g_29_0";
const string G_30_0 = "g_30_0";
const string G_31_0 = "g_31_0";
const string G_32_0 = "g_32_0";
const string G_33_0 = "g_33_0";
const string G_34_0 = "g_34_0";
const string G_35_0 = "g_35_0";
const string G_36_0 = "g_36_0";
const string G_37_0 = "g_37_0";
const string G_38_0 = "g_38_0";
const string G_39_0 = "g_39_0";
const string G_40_0 = "g_40_0";
const string G_41_0 = "g_41_0";
const string G_42_0 = "g_42_0";
const string G_43_0 = "g_43_0";
const string G_44_0 = "g_44_0";
const string G_45_0 = "g_45_0";
const string G_46_0 = "g_46_0";
const string G_47_0 = "g_47_0";
const string G_48_0 = "g_48_0";
const string G_49_0 = "g_49_0";
const string G_50_0 = "g_50_0";
const string G_51_0 = "g_51_0";
const string G_52_0 = "g_52_0";
const string G_53_0 = "g_53_0";
const string G_54_0 = "g_54_0";
const string G_55_0 = "g_55_0";
const string G_56_0 = "g_56_0";
const string G_57_0 = "g_57_0";
const string G_58_0 = "g_58_0";
const string G_59_0 = "g_59_0";
const string G_60_0 = "g_60_0";
const string G_61_0 = "g_61_0";
const string G_62_0 = "g_62_0";
const string G_63_0 = "g_63_0";
const string G_64_0 = "g_64_0";
const string G_65_0 = "g_65_0";
const string G_66_0 = "g_66_0";
const string G_67_0 = "g_67_0";
const string G_68_0 = "g_68_0";
const string G_69_0 = "g_69_0";
const string G_70_0 = "g_70_0";
const string G_71_0 = "g_71_0";
const string G_72_0 = "g_72_0";
const string G_73_0 = "g_73_0";
const string G_74_0 = "g_74_0";
const string G_75_0 = "g_75_0";
const string G_76_0 = "g_76_0";
const string G_77_0 = "g_77_0";
const string G_78_0 = "g_78_0";
const string G_79_0 = "g_79_0";
const string G_80_0 = "g_80_0";
const string G_81_0 = "g_81_0";
const string G_82_0 = "g_82_0";
const string G_83_0 = "g_83_0";
const string G_84_0 = "g_84_0";
const string G_85_0 = "g_85_0";
const string G_86_0 = "g_86_0";
const string G_87_0 = "g_87_0";
const string G_88_0 = "g_88_0";
const string G_89_0 = "g_89_0";
const string G_90_0 = "g_90_0";
const string G_91_0 = "g_91_0";
const string G_92_0 = "g_92_0";
const string G_93_0 = "g_93_0";
const string G_94_0 = "g_94_0";
const string G_95_0 = "g_95_0";
const string G_96_0 = "g_96_0";
const string G_97_0 = "g_97_0";
const string G_98_0 = "g_98_0";
const string G_99_0 = "g_99_0";
const string G_100_0 = "g_100_0";
const string G_101_0 = "g_101_0";
const string G_102_0 = "g_102_0";
const string G_103_0 = "g_103_0";
const string G_104_0 = "g_104_0";
const string G_105_0 = "g_105_0";
const string G_106_0 = "g_106_0";
const string G_107_0 = "g_107_0";
const string G_108_0 = "g_108_0";
const string G_109_0 = "g_109_0";
const string G_110_0 = "g_110_0";
const string G_111_0 = "g_111_0";
const string G_112_0 = "g_112_0";

// Palabras reservadas consideradas.
const int NUM_PALABRAS_RESERVADAS = 11;
const string reservadas[] = { 
  "boolean", "class", "const", "do", "int", "private", "protected",
  "public", "return", "void", "while"};

void Inicializar_Terminales(Terminales& ct);
void Inicializar_NoTerminales(NoTerminales& cnt);
void Inicializar_SimbolosSemanticos(SimbolosSemanticos& ss);
void Inicializar_SimbolosGenCodigo(SimbolosGenCodigo& sg);
void Inicializar_TablaAS(TablaAS& tas);

#endif // _Data_HPP_
